import time
import datetime
import calendar
from dateutil.relativedelta import relativedelta

from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.views.decorators.cache import cache_page
from django.db.models import Sum

import simplejson
from mywlcg import settings
from mywlcg.filters.lib import getname
from mywlcg.filters.views import parse_filters_dt
from mywlcg.mdb.models import (FtsHourlySummary, FtsDailySummary,
                               FtsWeeklySummary, FtsMonthlySummary)


def setdate(request, results):
    # Fixed dates
    if results.get('dateorperiod') == 'pd':
        fixed_period = results.get('period')
        if fixed_period == 'pD':
            delta = relativedelta(days=1)
            resolution = 'H'
        elif fixed_period == 'pW':
            delta = relativedelta(weeks=1)
            resolution = 'D'
        elif fixed_period == 'pM':
            delta = relativedelta(months=1)
            resolution = 'D'
        else:
            delta = relativedelta(months=6)
            resolution = 'M'

        enddate = datetime.datetime.utcnow()
        startdate = enddate - delta

    # Custom dates
    else:
        enddate = results.get('enddate')
        startdate = results.get('startdate')
        resolution = results.get('resolution')

        try:
            enddate = datetime.datetime(*time.strptime(enddate, '%d-%m-%Y')[0:6])
        except (ValueError, TypeError):
            enddate = datetime.datetime.utcnow()
        try:
            startdate = datetime.datetime(*time.strptime(startdate, '%d-%m-%Y')[0:6])
        except (ValueError, TypeError):
            startdate = enddate - relativedelta(months=1)

    # dates must be queried with exact datetime
    # adjust values depending on resolution
    data_startdate = startdate.replace(minute=0, second=0, microsecond=0)
    data_enddate = enddate.replace(minute=0, second=0, microsecond=0)
    if resolution == 'D':
        data_startdate = data_startdate.replace(hour=0)
        data_enddate = data_enddate.replace(hour=0)
    elif resolution == 'W':
        data_startdate -= relativedelta(days=data_startdate.weekday() + 1)
        data_enddate -= relativedelta(days=data_enddate.weekday() + 1)
        data_startdate = data_startdate.replace(hour=23)
        data_enddate = data_enddate.replace(hour=23)
    elif resolution == 'M':
        data_startdate = data_startdate.replace(day=1)
        data_enddate = data_enddate.replace(day=1)

    # dates for json call
    results['starttime'] = int(calendar.timegm(data_startdate.timetuple()))
    results['endtime'] = int(calendar.timegm(data_enddate.timetuple()))

    results['startdate'] = startdate.strftime("%d-%m-%Y")
    results['enddate'] = enddate.strftime("%d-%m-%Y")
    results['resolution'] = resolution
    return results


def index(request):
    seltab = "vo"
    return HttpResponseRedirect(reverse('mywlcg.dt.views.tab',kwargs={'tab':'vo'}))


def tab(request, tab):
    results={}
    results['tab']= str(tab)
    if tab != 'vo' and tab != 'sp' and tab != 'dp':
        return render_to_response('../../base/templates/400.html', {'error': 'Page Does Not Exist'})
    results['resolution']=str(request.GET.get('resolution') or 'H')
    results = parse_filters_dt(request, results)
    try:
        results = setdate(request,results)
    except ValueError, (ve):
        return render_to_response('../../base/templates/400.html', {'error': ve})
    return render_to_response('dt_index.html', results)


def heading(request, tab):
    view = int(request.GET.get("view") or 2) 
    resolution = str(request.GET.get("resolution") or "H")
    if view == 2:
        view_type='Average Data Throughput ';
    else:
        view_type='Aggregated Data Transfer ';
    if resolution=='H':
        resolution="Hourly "
    elif resolution=='D':
        resolution="Daily "
    elif resolution=='W':
        resolution="Weekly "
    else:
        resolution="Monthly "
    
    if tab == "vo":
        data= view_type+" "+ resolution +" Report (VO wise)"   
    elif tab == "sp":
        data= view_type+" "+resolution +" Report (Source Site wise)"
    elif tab == "dp":
        data= view_type+" "+resolution +" Report (Destination Site wise)"
    return render_to_response('dt_heading',{"data":data})


@cache_page(3600)
def json(request, tab):
    results = {}
    no_rows=0
    results['mapdata'] = {}
    results['mapdata']['others_display'] = 0
    void_list=[]
    srcsite_id=[]
    dstsite_id=[]
    others_str = ''
    others_line1 = ''
    namelist = []
    starttime = int(request.GET.get("starttime") or 0)
    endtime = int(request.GET.get("endtime") or 0)
    view=int(request.GET.get("view") or 2)
    resolution = str(request.GET.get("resolution") or 'H')
    s=datetime.datetime.utcfromtimestamp(starttime).date()
    e=datetime.datetime.utcfromtimestamp(endtime).date()
    if(s==e):
        endtime = int(calendar.timegm(s.timetuple())+24*3600-1)
        e=datetime.datetime.utcfromtimestamp(int(calendar.timegm(s.timetuple())+24*3600-1)) 
        resolution = 'H'
    start=str(s.strftime("%d-%m-%Y").lstrip("0"))
    end=str(e.strftime("%d-%m-%Y").lstrip("0"))
    diff=float(calendar.timegm(e.timetuple())-calendar.timegm(s.timetuple()))
    
    if view == 2:
        view_type = 'Average Throughput (MB/s)';
        view_type_short = 'Avg. Throughput'
        view_field = 'avgthroughput';
        div = 1000000.0
        unit = "MB/s"
    elif view == 3:
        view_type = 'Aggregated Data Transfer (TB) ';
        view_type_short = 'Aggregate Data '
        view_field = 'aggregatedata';
        div = 1000000000000.0
        unit = "TB"

    if resolution=='H':
        title=view_type+' on '+start
        xtitle='Time (UTC hour)'
        twidth=3600
    elif resolution=='D':
        title=view_type+' from '+start+' to '+ end
        xtitle='Date'
        twidth=86400
    elif resolution=='W':
        title=view_type+' from '+start+' to '+ end
        xtitle='Date'
        twidth=604800
    else:
        timediff = relativedelta(months=+1)
        title=view_type+' from '+start+' to '+ end
        xtitle='Date'
    
    if resolution=='H':
        resolution="Hourly"
        dateformat= "%H:%M"
    elif resolution=='D':
        resolution="Daily"
        dateformat= "%d-%m"
    elif resolution=='W':
        resolution="Weekly"
        dateformat= "%d-%m"
    else:
        resolution="Monthly"
        dateformat= "%m-%Y" 
    table = eval('Fts' + resolution + 'Summary')
    queryset = table.objects.filter(timestamp__range=(starttime,endtime))
    if queryset.count():
        results['gp'] =str(queryset.count())
        #vf_sum = queryset.aggregate(sum_vf = Sum(view_field)) 
    else:
        results['gp'] = ''
    if results['gp'] and tab =='vo':
        void_list = []
        vid_list = request.GET.getlist("vo") or ['ALL']
        if 'ALL' in vid_list:
            vo= queryset.values_list('void',flat=True).distinct()
            void_list = vo
        else:
            for value in vid_list:
                void_list.append(int(value))
        queryset = queryset.filter(void__in=void_list)
        if queryset.count():
            vf_sum = queryset.aggregate(sum_vf = Sum(view_field))
            vid_list = []
            vid_others_list = []
            #results['info'] = str(vf_sum) + "hi"
            vf_s = round(float(vf_sum['sum_vf']) * (float(settings.VO) / 100.0 ),0)            #vf_su = int(vf_s)
            vo_vf = queryset.values('void').annotate(sum_vo_vf = Sum(view_field)).order_by()
            for record in vo_vf:
                vo = record['void']
                v = int(vf_s - int(record['sum_vo_vf']))
                if v >= 0:
                    namelist.append(getname('Vo','voname',vo))
                    vid_others_list.append(vo)
                else:
                    vid_list.append(vo)
            others_str = ', '.join(namelist)
            tm_vf = queryset.values('timestamp').annotate(sum_tm_vf = Sum(view_field)).order_by()
            tm_vo_vf = queryset.values('timestamp','void').annotate(sum_tm_vo_vf = Sum(view_field)).order_by()
            time = starttime
            t=datetime.datetime.utcfromtimestamp(endtime)
            tmstr = str(t.strftime("%Y-%m-%dT%H:%M:%SZ"))
            i = 1
            results['mapdata']['rows'] = []
            j = 0
            while time <= endtime:
                no_rows = no_rows + 1
                total = 0.00
                t=datetime.datetime.utcfromtimestamp(time)
                tmstr = str(t.strftime(dateformat))
                recd = tm_vf.filter(timestamp=time)
                results['mapdata']['rows'].append([])
                results['mapdata']['rows'][j].append(tmstr)
                if recd:
                    value = 0.00
                    for vo in vid_others_list:
                        voname = getname("Vo","voname",vo)
                        vodata = tm_vo_vf.filter(timestamp=time,void=vo)
                        if vodata:
                            value = value + round(float(vodata[0]['sum_tm_vo_vf'])/div,2)
                    total = total + value
                    if len(vid_others_list):
                        results['mapdata']['rows'][j].append(round(value,2))
                    for vo in vid_list:
                        voname = getname("Vo","voname",vo)
                        vodata = tm_vo_vf.filter(timestamp=time,void=vo)
                        if vodata:
                            value = round(float(vodata[0]['sum_tm_vo_vf'])/div,2)
                            total = total + value
                            results['mapdata']['rows'][j].append(value)
                        else:
                            results['mapdata']['rows'][j].append('null')                    
                    results['mapdata']['rows'][j].append(round(total,2))
                else:
                    if len(vid_others_list):
                        results['mapdata']['rows'][j].append('null')
                    for vo in vid_list:
                        results['mapdata']['rows'][j].append('null')
                    results['mapdata']['rows'][j].append('null')
                j = j + 1
                if resolution == 'Monthly':
                    time2 = time + 28 * 24 * 3600 # Adding 28 days
                    t1=t.date()
                    t2=datetime.datetime.utcfromtimestamp(time2).date()
                    if t1.month == t2.month:
                        time2 = time2 + 4 * 24 * 3600 # 4 more days
                        t2=datetime.datetime.utcfromtimestamp(time2).date()
                    t2=datetime.datetime.utcfromtimestamp(calendar.timegm(t2.timetuple())-(t2.day-1)*24*3600)
                    time = int(calendar.timegm(t2.timetuple()))
                else:
                    time = time + twidth
            k = 0
            results['mapdata']['cols'] = []
            results['mapdata']['cols'].append({})
            results['mapdata']['cols'][0]['label'] = xtitle
            results['mapdata']['cols'][0]['type'] = "string"
            if len(vid_others_list):
                results['mapdata']['others_display'] = 0
                k = k + 1
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = str(len(vid_others_list)) + " Others "
                results['mapdata']['cols'][k]['type'] = "number"
            for vo in vid_list:
                k = k + 1
                voname = getname("Vo","voname",vo)
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = voname
                results['mapdata']['cols'][k]['type'] = "number"
            others_line1= "VOs with total " + view_type_short +" less than " + str(float(settings.VO)) + "% of total " + view_type_short + " within selected time range."
        else:
            results['gp'] = ''
    elif results['gp'] and tab == 'sp':
        srcsiteid_list = []
        dstsiteid_list = []
        srcstid_list = request.GET.getlist("srcsite") or ['ALL']
        dststid_list = request.GET.getlist("dstsite") or ['ALL']
   
        if 'ALL' in srcstid_list:
            srcsiteid_list= queryset.values_list('src_site_id',flat=True).distinct()
        else:
            for value in srcstid_list:
                srcsiteid_list.append(int(value))
        if 'ALL' in dststid_list:
            dstsiteid_list= queryset.values_list('dst_site_id',flat=True).distinct()
            queryset = queryset.filter(src_site_id__in=srcsiteid_list)
        else:
            for value in dststid_list:
                dstsiteid_list.append(int(value))
            queryset = queryset.filter(src_site_id__in=srcsiteid_list, dst_site_id__in= dstsiteid_list)
        if queryset.count():
            vf_sum = queryset.aggregate(sum_vf = Sum(view_field))
            srcsiteid_list = []
            srcsiteid_others_list = []
            vf_s  = round(float(vf_sum['sum_vf']) * (float(settings.SRCSITE)/ 100.0),0)
            srcsite_vf = queryset.values('src_site_id').annotate(sum_srcsite_vf = Sum(view_field)).order_by()
            for record in srcsite_vf:
                srcsite = record['src_site_id']
                v = int(vf_s - int(record['sum_srcsite_vf']))
                if v >= 0:
                    namelist.append(getname('Site','sitename',srcsite))
                    srcsiteid_others_list.append(srcsite)
                else:
                    srcsiteid_list.append(srcsite)
            others_str = ', '.join(namelist)
            #results['info'] = others_str
            tm_vf = queryset.values('timestamp').annotate(sum_tm_vf = Sum(view_field)).order_by()
            tm_src_vf = queryset.values('timestamp','src_site_id').annotate(sum_tm_src_vf = Sum(view_field)).order_by()
            time = starttime
            t=datetime.datetime.utcfromtimestamp(endtime)
            tmstr = str(t.strftime("%Y-%m-%dT%H:%M:%SZ"))
            i = 1
            results['mapdata'] = {}
            results['mapdata']['rows'] = []
            j = 0
            while time < endtime:
                no_rows = no_rows + 1
                total = 0.00
                t=datetime.datetime.utcfromtimestamp(time)
                tmstr = str(t.strftime(dateformat))
                recd = tm_vf.filter(timestamp=time)
                results['mapdata']['rows'].append([])
                results['mapdata']['rows'][j].append(tmstr)
                if recd:
                    value = 0.00
                    for site in srcsiteid_others_list:
                        sitename = getname("Site","sitename",site)
                        sitedata = tm_src_vf.filter(timestamp=time,src_site_id=site)
                        if sitedata:
                            value = value + round(float(sitedata[0]['sum_tm_src_vf'])/div,2)
                    total = total + value
                    if len(srcsiteid_others_list):
                        results['mapdata']['rows'][j].append(round(value,2))
                    for site in srcsiteid_list:
                        sitename = getname("Site","sitename",site)
                        sitedata = tm_src_vf.filter(timestamp=time,src_site_id=site)
                        if sitedata:
                            value = round(float(sitedata[0]['sum_tm_src_vf'])/div,2)
                            total = total + value
                            results['mapdata']['rows'][j].append(value)
                        else:
                            results['mapdata']['rows'][j].append('null')
                    results['mapdata']['rows'][j].append(round(total,2))
                else:
                    if len(srcsiteid_others_list):
                        results['mapdata']['rows'][j].append('null')
                    for site in srcsiteid_list:
                        results['mapdata']['rows'][j].append('null')
                    results['mapdata']['rows'][j].append('null')
                j = j + 1
                if resolution == 'Monthly':
                    time2 = time + 28 * 24 * 3600 # Adding 28 days
                    t1=t.date()
                    t2=datetime.datetime.utcfromtimestamp(time2).date()
                    if t1.month == t2.month:
                        time2 = time2 + 4 * 24 * 3600 # 4 more days
                        t2=datetime.datetime.utcfromtimestamp(time2).date()
                    t2=datetime.datetime.utcfromtimestamp(calendar.timegm(t2.timetuple())-(t2.day-1)*24*3600)
                    time = int(calendar.timegm(t2.timetuple()))
                else:
                    time = time + twidth
            k = 0
            results['mapdata']['cols'] = []
            results['mapdata']['cols'].append({})
            results['mapdata']['cols'][0]['label'] = xtitle
            results['mapdata']['cols'][0]['type'] = "string"
            if len(srcsiteid_others_list):
                results['mapdata']['others_display'] = 0
                k = k + 1
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = str(len(srcsiteid_others_list)) + " Others "
                results['mapdata']['cols'][k]['type'] = "number"
            for site in srcsiteid_list:
                k = k + 1
                sitename = getname("Site","sitename",site)
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = sitename
                results['mapdata']['cols'][k]['type'] = "number" 
            others_line1="Source Sites with total " + view_type_short +" less than " + str(float(settings.SRCSITE)) + "% of total " + view_type_short + " within selected time range."
        else:
            results['gp'] = ''
    elif results['gp'] and tab == 'dp':
        srcsiteid_list = []
        dstsiteid_list = []
        srcstid_list = request.GET.getlist("srcsite") or ['ALL']
        dststid_list = request.GET.getlist("dstsite") or ['ALL']
        if 'ALL' in dststid_list:
            dstsiteid_list= queryset.values_list('dst_site_id',flat=True).distinct()
        else:
            for value in dststid_list:
                dstsiteid_list.append(int(value))
        if 'ALL' in srcstid_list:
            queryset = queryset.filter(dst_site_id__in=dstsiteid_list)
        else:
            for value in srcstid_list:
                srcsiteid_list.append(int(value))
            queryset = queryset.filter(src_site_id__in=srcsiteid_list,dst_site_id__in=dstsiteid_list)
        if queryset.count():
            vf_sum = queryset.aggregate(sum_vf = Sum(view_field))
            dstsiteid_list = []
            dstsiteid_others_list = []
            #results['info'] = str(queryset.count())
            #results['info'] = settings.DSTSITE/100.0
            vf_s  = round(float(vf_sum['sum_vf']) * float(settings.DSTSITE/100.0),0)
            dstsite_vf = queryset.values('dst_site_id').annotate(sum_dstsite_vf = Sum(view_field)).order_by()
            for record in dstsite_vf:
                dstsite = record['dst_site_id']
                v = int(vf_s - int(record['sum_dstsite_vf']))
                if v >= 0:
                    namelist.append(getname('Site','sitename',dstsite))
                    dstsiteid_others_list.append(dstsite)
                else:
                    dstsiteid_list.append(dstsite)
            others_str = ', '.join(namelist)
            tm_vf = queryset.values('timestamp').annotate(sum_tm_vf = Sum(view_field)).order_by()
            tm_dst_vf = queryset.values('timestamp','dst_site_id').annotate(sum_tm_dst_vf = Sum(view_field)).order_by()
            time = starttime
            t=datetime.datetime.utcfromtimestamp(endtime)
            tmstr = str(t.strftime("%Y-%m-%dT%H:%M:%SZ"))
            i = 1
            results['mapdata'] = {}
            results['mapdata']['rows'] = []
            j = 0
            while time < endtime:
                no_rows = no_rows + 1
                total = 0.00
                t=datetime.datetime.utcfromtimestamp(time)
                tmstr = str(t.strftime(dateformat))
                recd = tm_vf.filter(timestamp=time)
                results['mapdata']['rows'].append([])
                results['mapdata']['rows'][j].append(tmstr)
                if recd:
                    value = 0.00
                    for site in dstsiteid_others_list:
                        sitename = getname("Site","sitename",site)
                        sitedata = tm_dst_vf.filter(timestamp=time,dst_site_id=site)
                        if sitedata:
                            value = value + round(float(sitedata[0]['sum_tm_dst_vf'])/div,2)
                    total = total + value
                    if len(dstsiteid_others_list):
                        results['mapdata']['rows'][j].append(round(value,2))
                    for site in dstsiteid_list:
                        sitename = getname("Site","sitename",site)
                        sitedata = tm_dst_vf.filter(timestamp=time,dst_site_id=site)
                        if sitedata:
                            value = round(float(sitedata[0]['sum_tm_dst_vf'])/div,2)
                            total = total + value
                            results['mapdata']['rows'][j].append(value)
                        else:
                            results['mapdata']['rows'][j].append('null')
                    results['mapdata']['rows'][j].append(round(total,2))
                else:
                    if len(dstsiteid_others_list):
                        results['mapdata']['rows'][j].append('null')
                    for site in dstsiteid_list:
                        results['mapdata']['rows'][j].append('null') 
                    results['mapdata']['rows'][j].append('null')
                j = j + 1
                if resolution == 'Monthly':
                    time2 = time + 28 * 24 * 3600 # Adding 28 days
                    t1=t.date()
                    t2=datetime.datetime.utcfromtimestamp(time2).date()
                    if t1.month == t2.month:
                        time2 = time2 + 4 * 24 * 3600 # 4 more days
                        t2=datetime.datetime.utcfromtimestamp(time2).date()
                    t2=datetime.datetime.utcfromtimestamp(calendar.timegm(t2.timetuple())-(t2.day-1)*24*3600)
                    time = int(calendar.timegm(t2.timetuple()))
                else:
                    time = time + twidth
            results['info'] = j
            k = 0
            results['mapdata']['cols'] = []
            results['mapdata']['cols'].append({})
            results['mapdata']['cols'][0]['label'] = xtitle
            results['mapdata']['cols'][0]['type'] = "string"
            if len(dstsiteid_others_list):
                results['mapdata']['others_display'] = 0
                k = k + 1
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = str(len(dstsiteid_others_list)) + " Others "
                results['mapdata']['cols'][k]['type'] = "number"    
            for site in dstsiteid_list:
                k = k + 1
                sitename = getname("Site","sitename",site)
                results['mapdata']['cols'].append({})
                results['mapdata']['cols'][k]['label'] = sitename
                results['mapdata']['cols'][k]['type'] = "number"  
                others_line1= "Destination Sites with total " + view_type_short +" less than " + str(float(settings.DSTSITE)) + "% of total " + view_type_short + " within selected time range."
        else:
            results['gp'] = ''
    else:
        results['gp'] = ''
    results['ytitle'] = view_type
    results['xtitle'] = xtitle
    results['mapdata']['unit'] = unit
    results['mapdata']['others_line1'] = others_line1
    results['mapdata']['others_list'] = others_str
    if no_rows < 12:
        results['mapdata']['slanted_text']= 0
        results['mapdata']['slanted_angle']= 0
    elif no_rows >= 12 and no_rows <= 24:
        results['mapdata']['slanted_text']= 1
        results['mapdata']['slanted_angle']= 0
    else:
        results['mapdata']['slanted_text']= 1
        results['mapdata']['slanted_angle']= 90
    results['info'] = no_rows
    data = simplejson.dumps(results)
    return HttpResponse(data, mimetype='application/json')
