from django.db.models.query_utils import Q
from django.core.exceptions import ObjectDoesNotExist
from mywlcg import settings
from mywlcg.mdb.models import *
import string

def interpolate(begin,end,val):
    if begin<end:
      return ((end-begin)*val/100)+begin
    return ((begin-end)*(1-(val/100)))+end

def colorgradient(graph,val):
    R0=0xFF0000 >>16
    G0=0x003000 >>8
    B0=0x000030
    if graph==1:
     R1=0x660000 >>16
     G1=0x00CD00 >>8
     B1=0x0
    else:
     R1=0x1E0000 >>16
     G1=0x009000 >>8
     B1=0x0000FF

    R=interpolate(R0, R1, val)
    G=interpolate(G0, G1, val)
    B=interpolate(B0, B1, val)
    ans1= str(hex((((int(R)<<8)|int(G))<<8)|int(B)))
    ans=ans1.split('x')
    ans=ans[1]
    last=6-len(ans)
    i=0
    while i<last:
         ans='0'+ans
         i=i+1
    return ans

def getname(table,field,value):
    result=[]
    result=eval(table).objects.filter(id=value).values(field)
    return str(result[0][field])

def parse_params(request):
    params = ""
    for key in request.GET:
        if key in ['profile', 'monitored', 'filters-value-Flavours', 'vo', 'status',\
            'filters-value-Hostnames', 'filters-value-Sites',\
            'filters-value-Regions', 'graph','view','sort','site']:
            for value in request.GET.getlist(key):
                if value != '':
                    params += "%s=%s&" % (key,value)
    if params != '':
        params = params[0:len(params)-1]
    return params

def get_profiles(vo):
    profiles = Profile.objects.filter(
        isdeleted='N', atp_vo=vo).order_by(
        'namespace__name', 'name').values_list(
        'id', 'name', 'namespace__name'
    )
    profiles = map(lambda x: (str(x[0]), x[2]+'-'+x[1]), profiles)
    return profiles

def get_vo_profile(request):
    vos = Profile.objects.filter(isdeleted='N').order_by('atp_vo__voname')
    vos = vos.values_list('atp_vo', 'atp_vo__voname').distinct()
    if request.GET.get("vo"):
        vo = int(request.GET.get("vo"))
    else: 
        vo = ''

    if request.GET.get("profile"):
        profile = int(request.GET.get("profile"))
    else:
        prof = Profile.objects.filter(name = settings.DEFAULT_PROFILE,isdeleted='N')\
			.order_by('namespace__name')
        if prof.count():
            profile = prof.values_list('id')[0][0]
        else:
            profile = 0

    if profile and not vo:
        vo = Profile.objects.filter(id=profile,isdeleted='N').values_list('atp_vo')[0][0]

    if not profile:
        profiles = get_profiles(vo)
	profile = profiles[0][0]

    return (vos,vo,profile)

def get_grouptype(vo, gtype='Site'):
    vo_gtype = "%s_%s" % (vo.upper(), gtype)
    try:
        vo_gtype = GroupType.objects.get(typename__iexact=vo_gtype, isdeleted='N')
    except ObjectDoesNotExist:
        return gtype
    return vo_gtype.typename

def parse_filters(request, queryset):
    # Selection
    vos,vo,profile=get_vo_profile(request)
    sitename = request.GET.get('sitename')
    regions = request.GET.get('filters-value-Regions')
    regions_list = []
    list=VoServiceGroup.objects
    if regions:
        regions_list = regions.split(',')
        if regions_list[len(regions_list)-1] == '':
            regions_list = regions_list[0:len(regions_list)-1]
        list1=list.filter(groups__in = regions_list).values_list('service',flat=True)
        queryset = queryset.filter(id__in = list1)
    sites = request.GET.get('filters-value-Sites')
    sites_list = []
    if sites:
        sites_list = sites.split(',')
        if sites_list[len(sites_list)-1] == '':
            sites_list = sites_list[0:len(sites_list)-1]
        list1=list.filter(groups__in=sites_list, isdeleted='N').values_list('service',flat=True)
        queryset = queryset.filter(id__in = list1)
    services = request.GET.get('filters-value-Hostnames')
    services_list = []
    if services:
        services_list = services.split(',')
        if services_list[len(services_list)-1] == '':
            services_list = services_list[0:len(services_list)-1]
        queryset = queryset.filter(id__in = services_list)

    # Common filters
    serv_flav = Profile.objects.filter(id = profile,isdeleted='N')[0].metricinstances\
                        .values_list('atp_service_type_flavour',flat='True')\
                        .distinct('atp_service_type_flavour')
    queryset = queryset.filter(flavour__in = serv_flav)
    monitored = int(request.GET.get("monitored", "1"))

    if monitored: 
        queryset = queryset.filter(ismonitored = 'Y')
    else:
        queryset = queryset.filter(ismonitored = 'N')

    flavours = request.GET.get("filters-value-Flavours")
    if flavours:
        flavours_list = flavours.split(',')
        if flavours_list[len(flavours_list)-1] == '':
            flavours_list = flavours_list[0:len(flavours_list)-1]
        queryset = queryset.filter(flavour__in = flavours_list)
    queryset = queryset.filter(vo = vo)
    if 'status' in request.GET:
        status = map(lambda x: int(x), request.GET.getlist('status'))
    else:
        status = MetricStatus.objects.exclude(description = 'REMOVED')\
            .values_list('id', flat=True)
    queryset = queryset.filter(\
        isdeleted='N',
        statuschangeserviceprofilelatest__profile = profile,
        statuschangeserviceprofilelatest__metricstatus__in = status)
    
    return [str(profile), queryset]
