import string
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import render_to_response
from django.views.decorators.cache import cache_page
from django.db.models.query_utils import Q

from mywlcg import settings
from mywlcg.base import release_info

from mywlcg.filters.lib import get_vo_profile, get_grouptype, get_profiles
from mywlcg.base.views import path_prefix
from mywlcg.mdb.models import (Profile, ServiceTypeFlavour, Vo, MetricStatus,
                               Service, Site, Country, Groups, GroupType, SupportedProfiles,
                               VoServiceGroup, AceSiteStatus)


def parse_selection(request, results):
    results['filters_summary'] = ''
    # Regions
    regions = request.GET.get("filters-value-Regions")
    if regions:
        regions_list = regions.split(',')
        if regions_list[len(regions_list)-1] == '':
            regions_list = regions_list[0:len(regions_list)-1]
        results['regions'] = Groups.objects.filter(id__in = regions_list)\
            .values_list('id', 'groupname')
        results['filters_summary'] += "<strong>Regions (%s):</strong>" % (len(results['regions']),)
        for region in results['regions']:
            results['filters_summary'] += " %s," % (region[1])
        results['filters_summary'] = results['filters_summary'][0:len(results['filters_summary'])-1] + '.<br />'
    else:
        results['regions'] = []
    # Sites
    sites = request.GET.get("filters-value-Sites")
    if sites:
        sites_list = sites.split(',')
        if sites_list[len(sites_list)-1] == '':
            sites_list = sites_list[0:len(sites_list)-1]
        results['sites'] = list(Groups.objects.filter(id__in = sites_list)\
            .values_list('id', 'groupname'))
        results['filters_summary'] += "<strong>Sites (%s):</strong>" % (len(results['sites']),)
        for site in results['sites']:
            results['filters_summary'] += " %s," % (site[1])
        results['filters_summary'] = results['filters_summary'][0:len(results['filters_summary'])-1] + '.<br />'
    else:
        results['sites'] = []
    # Services
    services = request.GET.get("filters-value-Hostnames")
    if services:
        services_list = services.split(',')
        if services_list[len(services_list)-1] == '':
            services_list = services_list[0:len(services_list)-1]
        results['services'] = list(Service.objects.filter(id__in = services_list)\
            .values_list('id', 'hostname', 'flavour__flavourname'))
        results['filters_summary'] += "<strong>Hostnames (%s):</strong>" % (len(results['services']),)
        for service in results['services']:
            results['filters_summary'] += " %s (%s)," % (service[1], service[2])
        results['filters_summary'] = results['filters_summary'][0:len(results['filters_summary'])-1] + '.<br />'
    else:
        results['services'] = []
    # Return
    return results

def parse_common_filters(request, results):
    path_prefix(request,results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    results['filters_summary'] += '<strong>Attributes</strong>:'
    results['vos'],results['vo'],results['profile']=get_vo_profile(request)
    results['filters_summary'] += " VO:" + Vo.objects.get(id = results['vo']).voname + ","
    prof = Profile.objects.filter(id = results['profile'],isdeleted='N').values_list('namespace__name', 'name')
    results['filters_summary'] += ' Profile: ' + prof[0][0] + '-' + prof[0][1] + ','
    results['monitored']= int(request.GET.get("monitored", "1"))
    results['removed']= int(request.GET.get("removed", "0"))
    results['production'] = int(request.GET.get("production", 1))
    results['preproduction'] = int(request.GET.get("preproduction", 1))
    results['statuses'] = MetricStatus.objects.values_list('id', 'description')
    results['iDisplayLength'] = int(request.GET.get('iDisplayLength') or 10)
    flavours = request.GET.get("filters-value-Flavours")
    if flavours:
        flavours_list = flavours.split(',')
        if flavours_list[len(flavours_list)-1] == '':
            flavours_list = flavours_list[0:len(flavours_list)-1]
        results['flavour'] = ServiceTypeFlavour.objects.filter(id__in = flavours_list)\
            .values_list('id', 'flavourname')
        results['filters_summary'] += "<strong>Regions (%s):</strong>" % (len(results['regions']),)
        for flavour in results['flavour']:
            results['filters_summary'] += " %s," % (flavour[1])
        results['filters_summary'] = results['filters_summary'][0:len(results['filters_summary'])-1] + '.<br />'
    else:
        results['flavour'] = []
    results['status'] = map(lambda x: int(x), request.GET.getlist('status'))
    if 'flavour' in results:
        if len(results['flavour']) == 1:
            results['filters_summary'] += " 1 flavour selected,"
        else:
            results['filters_summary'] += " %s flavours selected," % (len(results['flavour']),)
    if not 'status' in results:
        results['status'] = MetricStatus.objects\
            .exclude(description = 'REMOVED')\
            .values_list('id', flat=True)
    results['filters_summary'] += " %s status selected," % (len(results['status']),)
    if 'startdate' in results and results['startdate'] != '':
        results['filters_summary'] += " start date: %s," % (results['startdate'],)
    if 'enddate' in results and results['enddate'] != '':
        results['filters_summary'] += " end date: %s," % (results['enddate'],)
    results['filters_summary'] = results['filters_summary'][0:len(results['filters_summary'])-1] + '.'
    results['site']=0
    return results

####################################################


def authorized_dn(request):
    AUTHENTIC_DNS = settings.AUTHENTIC_DNS.split(',')
    DN = request.META.get('SSL_CLIENT_S_DN')
    if DN and DN in AUTHENTIC_DNS:
        return 1
    return 0


def _get_configured_report_profiles(report_name):
    data = getattr(settings, "%s_PROFILES" % report_name.upper(), '').split()
    for p in Profile.objects.filter(name__in=data):
        yield p


def parse_filters_reports(request, results, egi_only=False):
    path_prefix(request,results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    results['report']=int(request.GET.get("report") or 2)

    results['profiles']=[]
    for rdata in results['reports']:
        if egi_only and 'WLCG' in rdata:
            continue
        for p in _get_configured_report_profiles(rdata):
            if p not in results['profiles']:
                results['profiles'].append(p)

    results['profile'] = request.GET.get("profile")
    if results['profile'] != 'all':
        results['profile'] = int(results['profile'] or results['profiles'][0].id)

    results['vos'] = sorted(set([p.atp_vo for p in results['profiles']]), key=lambda v: v.voname)
    results['vo'] = request.GET.get('vo', 'all')

    results['wlcg_tier1_emails']=settings.WLCG_TIER1_EMAILS
    results['wlcg_tier2_emails']=settings.WLCG_TIER2_EMAILS.split()
    results['wlcg_egi_emails']=settings.EGI_EMAILS.split()
    results['report']=int(request.GET.get("report") or 2)
    results['history']=int(request.GET.get("history") or 6)
    results['months']=['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']
    
    return results

####################################################

def parse_filters_dt(request,results):
    path_prefix(request,results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    results['profile']=0
    if results['tab'] == 'vo':
      void=request.GET.get('filters-value-VOs')
      if void:
        voids_list = void.split(',')
        if voids_list[len(voids_list)-1] == '':
            voids_list = voids_list[0:len(voids_list)-1]
        results['void'] = Vo.objects.filter(id__in = voids_list)\
            .values_list('id', 'voname')
      
        results['voslist']=''
        for val in voids_list:
           results['voslist']+='&vo='+val
      else:
        results['voslist']='&vo=ALL'
    elif results['tab'] == 'sp' or results['tab'] == 'dp':
      srcsite =request.GET.get('filters-value-Source_Sites')
      dstsite =request.GET.get('filters-value-Destination_Sites')
      if srcsite:
        srcsites_list = srcsite.split(',')
        if srcsites_list[len(srcsites_list)-1] == '':
            srcsites_list = srcsites_list[0:len(srcsites_list)-1]
        results['srcsite'] = Site.objects.filter(id__in = srcsites_list)\
            .values_list('id', 'sitename')
        results['srcsiteslist']=''
        for val in srcsites_list:
           results['srcsiteslist']+='&srcsite='+val
      else:
        results['srcsiteslist']='&srcsite=ALL'
      if dstsite:
        dstsites_list = dstsite.split(',')
        if dstsites_list[len(dstsites_list)-1] == '':
            dstsites_list = dstsites_list[0:len(dstsites_list)-1]
        results['dstsite'] = Site.objects.filter(id__in = dstsites_list)\
            .values_list('id', 'sitename')
        results['dstsiteslist']=''
        for val in dstsites_list:
           results['dstsiteslist']+='&dstsite='+val
      else:
        results['dstsiteslist']='&dstsite=ALL'
    results['view']=int(request.GET.get("view") or 2)
    results['dateorperiod']=str(request.GET.get("dateorperiod") or "pd")
    results['period']=str(request.GET.get("period") or 'pD')
    if results['view'] == 2:
        results['view_type']='Average Throughput ';
    else:
        results['view_type']='Aggregated Data Transfer ';
    results['startdate'] = request.GET.get('startdate')
    results['enddate'] = request.GET.get('enddate')
    return results	

####################################################

def parse_filters_trends(request, results):
    path_prefix(request, results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    vos, vo, profile = get_vo_profile(request)
    results['vos'] = vos
    req_profile = request.GET.get('profile')

    try:
        prof = Profile.objects.get(id=req_profile)
    except ObjectDoesNotExist:
        prof = Profile.objects.get(name='WLCG_CREAM_LCGCE_CRITICAL')

    results['vo'] = prof.atp_vo.id
    results['profile'] = prof.id

    results['view'] = request.GET.get("view") or 1
    results['bestn'] = int(request.GET.get("bestn") or 20)
    results['bestm'] = int(request.GET.get("bestm") or 50)

    results['plottype'] = request.GET.get("plottype") or "b"
    if results['plottype'] == 'b' and results['tab'] == 'eu':
       results['borl'] = "ColumnChart"
    else:
       results['borl'] = "LineChart"

    results['view'] = int(request.GET.get("view") or 1)
    results['group'] = request.GET.get("group") or 'tier0'
    results['site'] = request.GET.get("site") or ''

    # dates
    results['dateorperiod'] = request.GET.get('dateorperiod', 'pd')
    results['period'] = request.GET.get("period", "p6")
    results['startdate'] = request.GET.get("startdate")
    results['enddate'] = request.GET.get("enddate")

    return results

####################################################

def parse_filters_sa(request, results):
    path_prefix(request,results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    results['vos'],results['vo'],results['profile']=get_vo_profile(request)
    results['site']=int(request.GET.get("filters-value-Sites") or 0)
    results['view']=int(request.GET.get("view") or 2)
    results['sort']=int(request.GET.get("sort") or 3)
    results['graph']=int(request.GET.get("graph") or 1)
    results['service']=int(request.GET.get("service") or 0)
    results['service_type_flavour']=int(request.GET.get("service_type_flavour") or 0)
    results['service_type_flavour_name']=request.GET.get("service_type_flavour_name") or 0
    results['service_name']=request.GET.get("service_name") or 0
    results['site_name']=request.GET.get("site_name") or 0
    results['resolution']=request.GET.get("resolution") or 'D'
    results['dateorperiod']=str(request.GET.get("dateorperiod") or "pd")
    results['period']=str(request.GET.get("period") or 'pM')
    results['production']=int(request.GET.get('production') or 1)
    results['preproduction']=int(request.GET.get('preproduction') or 1)
    groups = request.GET.get("filters-value-Regions_or_Tiers")
    if groups:
        groups_list = groups.split(',')
        if groups_list[len(groups_list)-1] == '':
            groups_list = groups_list[0:len(groups_list)-1]
        results['groups'] = Groups.objects.filter(id__in = groups_list)\
            .values_list('id', 'groupname')
        results['grouplen']=1
        results['groupslist']=groups
    else:
        results['grouplen']=0
        results['groupslist']=''

    results['sites']=Groups.objects.filter(id = results['site']).values_list('id', 'groupname')

    if results['graph']==1:
        results['graphtype'] = 'Availability';
    else:
        results['graphtype'] = 'Reliability';

    groupsname=''
    if results['grouplen']:
        i=0 
        for gname in results['groups']:
            if i>0:
               groupsname = groupsname + ', '
            groupsname = groupsname + gname[1]
            i = i + 1
        results['filters_summary'] = '<b>Groups(' + str(len(results['groups'])) + '): </b>' + groupsname
    if results['site']:
       if groupsname:
          results['filters_summary'] = results['filters_summary'] + ', '
          results['filters_summary'] = results['filters_summary'] + '<b>Site:</b> '\
		+ Groups.objects.get(id=results['site']).groupname
    return results
    
####################################################

def parse_filters_gridmap(request, results):
    path_prefix(request,results)
    info = release_info()
    results['about'] = info['sam_release']
    results['version'] = info['mywlcg_version']

    results['dview'] = request.GET.get("dview") or 'Status'
    results['dviews'] = []
    if settings.ACE:
       results['dviews'] = ['Status','Availability']
    results['vos'],results['vo'],results['profile']=get_vo_profile(request)
    results['site']=0
    results['show'] = int(request.GET.get("show") or 0)
    results['autoupdate'] = int(request.GET.get("autoupdate") or 0)
    if request.GET.get("groupname"):
        results['groupname'] = request.GET.get("groupname")
        gname=[results['groupname'],]
    elif settings.REGION:
        results['groupname'] = settings.REGION
        gname=results['groupname'].split()
    else:
        results['groupname']=''
    
    results['sitename'] = request.GET.get("sitename") or ''

    results['grouptypes'] = GroupType.objects.filter(typename__in=['Region','Tier'])\
				.values_list('id', 'typename')

    results['grouptype'] = int(request.GET.get("grouptype") or GroupType.objects.get(typename='Region').id)
    
    results['flavours'] =  ServiceTypeFlavour.objects.filter(isdeleted='N')\
					.values_list( 'id', 'flavourname')\
					.order_by('flavourname').distinct() 

    results['sflavour'] = int(request.GET.get("sflavour") or 0)
    
    if results['groupname']:
       group = Groups.objects.filter(groupname__in = gname, isdeleted='N', group_type__typename='Region')\
           		.values_list('description', flat = True)
       if len(group):
          results['description'] = group[0]
          i=1
          while i<len(group):
             results['description'] = results['description'] + ', ' + group[i]
             i=i+1
       else:
           results['description'] = Groups.objects.get(groupname = results['groupname'], isdeleted='N', 
					group_type__typename='Tier').description
    else:
       results['description'] = GroupType.objects.get(id=results['grouptype']).typename
    
    results['size'] = int(request.GET.get("size") or 1)
    results['havail'] = request.GET.get("havail") or 'D'
   
    prof = Profile.objects.filter(id = results['profile'],isdeleted='N').values_list('namespace__name','name')
    results['filters_summary'] = ' Profile: ' + prof[0][0] + '-' + prof[0][1] 
    if results['sflavour']:
	results['filters_summary'] = results['filters_summary'] + ', Flavour: '\
			 + ServiceTypeFlavour.objects.get(id =results['sflavour']).flavourname
    return results

######################################################

@cache_page(3600)
def json_group(request,type, vo):
     q = request.GET.get("q") or ''
     if int(vo)>0:
         voname = Vo.objects.get(id=vo).voname
     if type == "Countries":
         queryset = Country.objects
         if len(q):
            queryset = queryset.filter(countryname__icontains = q)
         results = queryset.values_list('id', 'countryname').order_by('countryname').distinct()
         return render_to_response('filters_json', {'results': results})
     elif type == "Groups":
         queryset = VoServiceGroup.objects.filter(isdeleted='N',groups__isdeleted='N',vo__voname=voname)
         if len(q):
            queryset = queryset.filter(groups__groupname__icontains = q)
         results = queryset.values_list('groups', 'groups__groupname').order_by('groups__groupname').distinct()
         return render_to_response('filters_json', {'results': results})
     elif type == "Group_Types":
         queryset = VoServiceGroup.objects.filter(isdeleted='N',groups__group_type__isdeleted='N'
							,vo__voname=voname)
         if len(q):
            queryset = queryset.filter(groups__group_type__typename__icontains = q)
         results = queryset.values_list('groups__group_type', 'groups__group_type__typename')\
				.order_by('groups__group_type__typename').distinct()
         return render_to_response('filters_json', {'results': results})
     elif type == "Regions_or_Tiers":
         type = get_grouptype(voname,'Tier')
         filter = Q(group_type__typename__iexact='Region')| Q(group_type__typename__iexact=type)
         queryset = Groups.objects.filter(filter, isdeleted='N')
         if len(q):
            queryset = queryset.filter(groupname__icontains = q)
         results = queryset.values_list('id', 'groupname').order_by('groupname').distinct()
         return render_to_response('filters_json', {'results': results})
     elif type == "Sites":
         gtype = get_grouptype(voname)
         queryset = Groups.objects.filter(group_type__typename__iexact=gtype,isdeleted='N')
         if len(q):
            queryset = queryset.filter(groupname__icontains = q)
         results = queryset.values_list('id', 'groupname').order_by('groupname').distinct()
         return render_to_response('filters_json', {'results': results})
     elif type == "profiles":
         results = get_profiles(vo)
         return render_to_response('filters_json', {'results': results})
     elif type == "Hostnames":
        list = VoServiceGroup.objects.filter(vo=vo).values_list('service')\
			.order_by('service').distinct()
        queryset = Service.objects.filter(id__in=list,
            statuschangeserviceprofilelatest__isnull = False).distinct()
        if len(q):
            queryset = queryset.filter(hostname__icontains = q)
        results = queryset.values_list('id', 'hostname', 'flavour__flavourname').order_by('hostname').distinct()
        return render_to_response('filters_json_services', {'results': results})
     elif type == "Regions":
        queryset = Groups.objects.filter(group_type__typename = 'Region')
        if len(q):
            queryset = queryset.filter(groupname__icontains = q)
        results = queryset.values_list('id', 'groupname').order_by('groupname').distinct()
        return render_to_response('filters_json', {'results': results})
     elif type == "Flavours":
        queryset = ServiceTypeFlavour.objects
        if len(q):
            queryset = queryset.filter(flavourname__icontains = q)
	results = queryset.values_list('id', 'flavourname').order_by('flavourname').distinct()
	return render_to_response('filters_json', {'results': results})
     elif type == "VOs":
        queryset = Vo.objects
        if len(q):
           queryset = queryset.filter(voname__icontains = q)
	results = queryset.values_list('id', 'voname').order_by('voname').distinct() 
        return render_to_response('filters_json', {'results': results})
     elif type in ["Source_Sites","Destination_Sites","Physical_Sites"]:
        queryset = Site.objects
	if len(q):
           queryset = queryset.filter(sitename__icontains = q)
        results =queryset.values_list('id','sitename').order_by('sitename').distinct()
        return render_to_response('filters_json', {'results': results})
