from django.db import models
from django.db.models import Model
from atp.orm.models import *
from Poem_sync.poem_sync.models import *

#####################################################
# NOTE : While addind a new table/view to this file,
# please add the new classname to tables_list
# array defined in ../tests/tests.py file, so that
# the table could be tested.
#####################################################


YESNO_CHOICE= ((u'Y',u'Yes'), (u'N', u'No'),)

# New
class Gatheredat(models.Model):
    id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=256, blank=True)
    class Meta:
        db_table = u'gatheredat'

#####################################################
# MRS
#####################################################
class Fqan(Model):
    id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=768)
    class Meta:
        db_table = u'fqan'

class SupportedProfiles(Model):
    id = models.IntegerField(primary_key=True)
    name =  models.CharField(max_length=256)
    INSERT_TIME = models.IntegerField()
    class Meta:
        db_table = u'supported_profiles'
	ordering = ('name',)

class Service_Site(Model):
    id = models.AutoField(primary_key=True)
    service = models.ForeignKey(Service)
    site=models.ForeignKey(Site)
    isdeleted=models.CharField(max_length=1,choices= YESNO_CHOICE, default='N')
    class Meta:
        db_table = u'service_site'


class MetricStatus(Model):
    id=models.AutoField(primary_key=True)
    description = models.CharField(unique=True, max_length=45, blank=True)
    description_nagios = models.CharField(max_length=45, blank=True)
    comments = models.CharField(max_length=45, blank=True)
    class Meta:
        db_table = u'metricstatus'
    def __unicode__(self):
        return self.description

class MetricDetails(Model):
     id=models.AutoField(primary_key=True)
     detail=models.TextField(null=True,blank=True)
     class Meta:
        db_table = u'metricdetails'
     def __unicode__(self):
        return self.detail

class MetricData(Model):
     metricstatus=models.ForeignKey(MetricStatus,db_column="metricstatus_id")
     metricdetail=models.ForeignKey(MetricDetails,db_column="metricdetail_id")
     metric=models.ForeignKey(Metric,db_column="poem_sync_metric_id")
     service=models.ForeignKey(Service,db_column="service_id")
     vo=models.ForeignKey(Vo,db_column="vo_id")
     summarydata=models.CharField( max_length=256)
     gatheredat = models.ForeignKey(Gatheredat,db_column='gatheredat_id')
     fqan = models.ForeignKey(Fqan, db_column="fqan_id")
     check_time=models.IntegerField(unique=True)
     insert_time=models.IntegerField(unique=True)
     class Meta:
        db_table = u'metricdata'
        unique_together = ("metricstatus", "metric","service","vo","check_time")
     def __unicode__(self):
        return self.summarydata

class StatusChangeServiceProfile(Model):
    service=models.ForeignKey(Service,db_column="service_id")
    profile=models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    metricstatus=models.ForeignKey(MetricStatus,db_column="metricstatus_id")
    detail=models.TextField("detail", max_length=256)
    timestamp=models.IntegerField(unique=True)
    class Meta:
        db_table = u'statuschange_service_profile'
        unique_together = ("service", "profile","metricstatus","timestamp")
        ordering = ('service__hostname','-timestamp',)
    def __unicode__(self):
        return self.detail

class StatusChangeServiceProfileLatest(Model):
    id=models.AutoField(primary_key=True)
    service=models.ForeignKey(Service,db_column="service_id")
    profile=models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    metricstatus=models.ForeignKey(MetricStatus,db_column="metricstatus_id")
    detail=models.TextField("detail", max_length=256)
    timestamp=models.IntegerField(unique=True)
    #def save(self):
    #    self.profile = self.p_profile
    #    if m_profile:
    #        self.profile = self.m_profile
    #    super(StatusChangeServiceProfileLatest, self).save()
    class Meta:
        db_table = u'statuschange_service_profile_l'
        unique_together = ("service", "profile","metricstatus","timestamp")
    def __unicode__(self):
        return self.detail

class StatusChangeMetricService(Model):
    id=models.AutoField(primary_key=True)
    metric=models.ForeignKey(Metric,db_column="poem_sync_metric_id")
    service=models.ForeignKey(Service,db_column="service_id")
    vo=models.ForeignKey(Vo,db_column='vo_id')
    metricstatus=models.ForeignKey(MetricStatus,db_column="metricstatus_id")
    timestamp=models.IntegerField(unique=True)
    class Meta:
        db_table = u'statuschange_metric_service'
        unique_together = ("service", "metric", "metricstatus", "vo", "timestamp")
    def __unicode__(self):
        return self.id

class MetricDataLatest(Model):
    id = models.AutoField(primary_key=True)
    metricstatus = models.ForeignKey(MetricStatus,db_column="metricstatus_id")
    metricdetail = models.ForeignKey(MetricDetails,db_column="metricdetail_id")
    metric = models.ForeignKey(Metric,db_column="poem_sync_metric_id")
    service = models.ForeignKey(Service,db_column="service_id")
    vo = models.ForeignKey(Vo,db_column="vo_id")
    summarydata = models.CharField(max_length=256, blank=True)
    gatheredat = models.ForeignKey(Gatheredat,db_column='gatheredat_id')
    fqan = models.ForeignKey(Fqan, db_column="fqan_id")
    check_time = models.IntegerField(unique=True)
    insert_time = models.IntegerField(unique=True)
    class Meta:
        db_table = u'metricdata_latest'
        unique_together=("metricstatus","metric","service","vo","check_time")
    def __unicode__(self):
        return self.summarydata

class ServiceDowntime(Model):
    id = models.AutoField(primary_key=True)
    service = models.ForeignKey(Service)
    downtime = models.ForeignKey(Downtime)
    isdeleted = models.CharField(max_length=1)
    class Meta:
        db_table = u'service_downtime'

class MetricstoreCurrLoadM(Model):
    id = models.AutoField(primary_key=True)
    timestamp = models.DateTimeField()
    hour = models.IntegerField()
    minute = models.IntegerField()
    number_of_records = models.IntegerField()
    class Meta:
        db_table = u'metricstore_curr_load_m'

class MetricstoreCurrLoadH(Model):
    id = models.AutoField(primary_key=True)
    rec_date = models.CharField(max_length=20)
    timestamp = models.DateTimeField()
    hour = models.IntegerField()
    number_of_records = models.IntegerField()
    class Meta:
        db_table = u'metricstore_curr_load_h'

class MetricstoreCurrLoadHDET(Model):
    id = models.AutoField(primary_key=True)
    rec_date = models.CharField(max_length=20)
    timestamp = models.DateTimeField()
    hour = models.IntegerField()
    ngi_name = models.CharField(max_length=20)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    flavour = models.ForeignKey(ServiceTypeFlavour)
    number_of_records = models.IntegerField()
    class Meta:
        db_table = u'metricstore_curr_load_h_det'

class MetricdataSpool(Model):
    id = models.AutoField(primary_key=True)
    class Meta:
        db_table = u'metricdata_spool'

class MetricdataForRecalculation(Model):
    id = models.AutoField(primary_key=True)
    class Meta:
        db_table = u'metricdataforrecalculation'

#####################################################
# ACE
#####################################################

class AceSiteStatus(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    grouping = models.ForeignKey(Groups)
    metricstatus = models.ForeignKey(MetricStatus)
    downtime_flag = models.IntegerField()
    timestamp = models.DateTimeField(db_column='start_timestamp',unique=True)
    end_timestamp = models.DateTimeField()
    class Meta:
        db_table = u'ace_sitestatus'
        ordering = ('grouping',)

class AceSiteStatusLatest(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    grouping = models.ForeignKey(Groups)
    metricstatus = models.ForeignKey(MetricStatus)
    downtime_flag = models.IntegerField()
    start_timestamp = models.DateTimeField(unique=True)
    end_timestamp = models.DateTimeField()
    class Meta:
        db_table = u'ace_sitestatus_latest_mv'
        ordering = ('grouping',)

class AceFlavourStatus(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    grouping = models.ForeignKey(Groups)
    service_type_flavour = models.ForeignKey(ServiceTypeFlavour)
    metricstatus = models.ForeignKey(MetricStatus)
    downtime_flag = models.IntegerField()
    timestamp = models.DateTimeField(db_column='start_timestamp',unique=True)
    end_timestamp = models.DateTimeField()
    class Meta:
        db_table = u'ace_flavourstatus'
        ordering = ('grouping','service_type_flavour__flavourname',)

class AceServiceStatus(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    service = models.ForeignKey(Service)
    metricstatus = models.ForeignKey(MetricStatus)
    downtime_flag = models.IntegerField()
    start_timestamp = models.DateTimeField(unique=True)
    end_timestamp = models.DateTimeField()
    class Meta:
        db_table = u'ace_servicestatus'

class AceServiceStatusLatest(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    service = models.ForeignKey(Service)
    metricstatus = models.ForeignKey(MetricStatus)
    downtime_flag = models.IntegerField()
    start_timestamp = models.DateTimeField(unique=True)
    end_timestamp = models.DateTimeField()
    class Meta:
        db_table = u'ace_servicestatus_latest_mv'

class AceAlgorithm(Model):
    id = models.AutoField(primary_key=True)
    site_algo = models.CharField(max_length=500)
    service_algo = models.CharField(max_length=500)
    class Meta:
        db_table = u'ace_algorithm'

class AceAlgorithmProfileMap(Model):
    id = models.AutoField(primary_key=True)
    profile = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    algorithm = models.ForeignKey(AceAlgorithm)
    class Meta:
        db_table = u'ace_algorithm_profile_map'

class AceAvailabilityTask(Model):
    id = models.AutoField(primary_key=True)
    group = models.ForeignKey(Groups)
    start_timestamp = models.DateTimeField(unique=True)
    end_timestamp = models.DateTimeField()
    considered = models.CharField(max_length=1)
    class Meta:
        db_table = u'ace_availability_tasks'

class ACE_H_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    service = models.ForeignKey(Service)
    class Meta:
        db_table = u'ACE_H_AVAILABILITY'
        verbose_name = u'ACE_H_AVAILABILITY'
        verbose_name_plural = u'ACE_H_AVAILABILITYs'
        ordering = ('service__flavour','service__hostname',)

class ACE_D_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    service = models.ForeignKey(Service)
    class Meta:
        db_table = u'ACE_D_AVAILABILITY'
        verbose_name = u'ACE_D_AVAILABILITY'
        verbose_name_plural = u'ACE_D_AVAILABILITYs'
        ordering = ('service__flavour','service__hostname',)

class ACE_W_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    service = models.ForeignKey(Service)
    class Meta:
        db_table = u'ACE_W_AVAILABILITY'
        verbose_name = u'ACE_W_AVAILABILITY'
        verbose_name_plural = u'ACE_W_AVAILABILITYs'
        ordering = ('service__flavour','service__hostname',)

class ACE_M_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    service = models.ForeignKey(Service)
    class Meta:
        db_table = u'ACE_M_AVAILABILITY'
        verbose_name = u'ACE_M_AVAILABILITY'
        verbose_name_plural = u'ACE_M_AVAILABILITYs'
        ordering = ('service__flavour','service__hostname',)

class ACE_H_FLAVOUR_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    SERVICE_TYPE_FLAVOUR = models.ForeignKey(ServiceTypeFlavour)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_H_FLAVOUR_AVAILABILITY'
        verbose_name = u'ACE_H_FLAVOUR_AVAILABILITY'
        verbose_name_plural = u'ACE_H_FLAVOUR_AVAILABILITYs'
        ordering = ('SERVICE_TYPE_FLAVOUR__flavourname',)

class ACE_D_FLAVOUR_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    SERVICE_TYPE_FLAVOUR = models.ForeignKey(ServiceTypeFlavour)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_D_FLAVOUR_AVAILABILITY'
        verbose_name = u'ACE_D_FLAVOUR_AVAILABILITY'
        verbose_name_plural = u'ACE_D_FLAVOUR_AVAILABILITYs'
        ordering = ('SERVICE_TYPE_FLAVOUR__flavourname',)

class ACE_W_FLAVOUR_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    SERVICE_TYPE_FLAVOUR = models.ForeignKey(ServiceTypeFlavour)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_W_FLAVOUR_AVAILABILITY'
        verbose_name = u'ACE_W_FLAVOUR_AVAILABILITY'
        verbose_name_plural = u'ACE_W_FLAVOUR_AVAILABILITYs'
        ordering = ('SERVICE_TYPE_FLAVOUR__flavourname',)

class ACE_M_FLAVOUR_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    SERVICE_TYPE_FLAVOUR = models.ForeignKey(ServiceTypeFlavour)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_M_FLAVOUR_AVAILABILITY'
        verbose_name = u'ACE_M_FLAVOUR_AVAILABILITY'
        verbose_name_plural = u'ACE_M_FLAVOUR_AVAILABILITYs'
        ordering = ('SERVICE_TYPE_FLAVOUR__flavourname',)


class ACE_H_SITE_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_H_SITE_AVAILABILITY'
        verbose_name = u'ACE_H_SITE_AVAILABILITY'
        verbose_name_plural = u'ACE_H_SITE_AVAILABILITYs'
        ordering = ('GROUPING__groupname',)

class ACE_D_SITE_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)#models.IntegerField()
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_D_SITE_AVAILABILITY'
        verbose_name = u'ACE_D_SITE_AVAILABILITY'
        verbose_name_plural = u'ACE_D_SITE_AVAILABILITYs'
        ordering = ('GROUPING__groupname',)

class ACE_W_SITE_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_W_SITE_AVAILABILITY'
        verbose_name = u'ACE_W_SITE_AVAILABILITY'
        verbose_name_plural = u'ACE_W_SITE_AVAILABILITYs'
        ordering = ('GROUPING__groupname',)

class ACE_M_SITE_AVAILABILITY(Model):
    id = models.AutoField(primary_key=True)
    TIMESTAMP  =  models.DateTimeField()
    GROUPING = models.ForeignKey(Groups)
    PROFILE = models.ForeignKey(Profile,db_column="poem_sync_profile_id")
    AUP = models.FloatField()
    ADOWN = models.FloatField()
    AMAINTENANCE = models.FloatField()
    AUNKNOWN = models.FloatField()
    AVAILABILITY = models.FloatField()
    DOWN = models.FloatField()
    MAINTENANCE = models.FloatField()
    UNKNOWN = models.FloatField()
    RELIABILITY = models.FloatField()
    class Meta:
        db_table = u'ACE_M_SITE_AVAILABILITY'
        verbose_name = u'ACE_M_SITE_AVAILABILITY'
        verbose_name_plural = u'ACE_M_SITE_AVAILABILITYs'
        ordering = ('GROUPING__groupname',)

class T0T1SERVICETARGETS(Model):
    availtargetpersite = models.FloatField()
    reltargetpersite = models.FloatField()
    availtargetbestsites = models.FloatField()
    reltargetbestsites = models.FloatField()
    effectivefromts = models.DateTimeField()
    modifiedts = models.DateTimeField()
    class Meta:
        db_table = u'T0T1SERVICETARGETS'


class TopologyView(Model):
    group_id = models.IntegerField()
    groupname = models.CharField(max_length=100)
    site_id = models.IntegerField()
    sitename = models.CharField(max_length=100)
    profile = models.CharField(max_length=255)
    voname = models.CharField(max_length=100)
    tier = models.CharField(max_length=100)
    region = models.CharField(max_length=100)
    certifstatus = models.CharField(max_length=128)
    infrasttype = models.CharField(max_length=128)
    country = models.CharField(max_length=100)
    federation = models.CharField(max_length=255)
    accounting_federation = models.CharField(max_length=100)
    phy_cpu_count = models.IntegerField()
    log_cpu_count = models.IntegerField()
    hspec06 = models.IntegerField()

    class Meta:
        db_table = u'Topology_View'


class SnapshotTopologyView(Model):
    loggroupingid = models.IntegerField()
    logsitename = models.CharField(max_length=100)
    phygroupingid = models.IntegerField()
    physitename = models.CharField(max_length=100)
    profilename = models.CharField(max_length=255)
    voname = models.CharField(max_length=100)
    tier = models.CharField(max_length=100)
    region = models.CharField(max_length=100)
    certifstatus = models.CharField(max_length=128)
    infrasttype = models.CharField(max_length=128)
    countryname = models.CharField(max_length=100)
    fedname = models.CharField(max_length=100)
    fedacname = models.CharField(max_length=255)
    phycpucount = models.IntegerField()
    logcpucount = models.IntegerField()
    hspec06 = models.IntegerField()
    snapshotts = models.DateTimeField()

    class Meta:
        db_table = u'"SNAPSHOT_TOPOLOGY_VIEW"'


####################################################
# SAM-1289 MRS - new view 'FQANS_IN_REGIONS'
#####################################################

class FqansInRegions(Model):
    roc_name = models.CharField("ROC Name", max_length=255)
    fqan = models.ForeignKey(Fqan)
    class Meta:
        db_table = u'fqans_in_regions'
        
class MetricHistoryInProfile(Model):
    profile_name = models.CharField(max_length=255)
    namespace = models.CharField(max_length=200)
    defined_by_vo = models.CharField(max_length=100)
    hostname = models.CharField(max_length=512)
    service_flavour = models.CharField(max_length=128)
    metric_exec_time = models.IntegerField()
    metric_name = models.CharField(max_length=255)
    fqan = models.CharField(max_length=256)
    tested_by_vo = models.CharField(max_length=8)
    metric_status = models.CharField(max_length=45, null=True, blank=True)
    metric_output_summary = models.CharField(max_length=256, null=True, blank=True)
    gathered_at = models.CharField(max_length=256, null=True, blank=True)
    metric_result_insert_time = models.IntegerField()
    class Meta:
        db_table = u'metric_history_in_poem_s_prof'
        verbose_name = u'metric_history_in_poem_s_prof'
        unique_together= ('profile_name','hostname')
        ordering = ('metric_name', )
#####################################################################################
# Models for FTS Visualization

class FtsHourlySummary(Model):
    timestamp = models.IntegerField(null=False, primary_key=True)
    void = models.IntegerField(null=False, primary_key=True)
    src_site_id = models.IntegerField(null=False, primary_key=True)
    dst_site_id = models.IntegerField(null=False, primary_key=True)
    aggregatedata = models.DecimalField(max_digits=10, decimal_places=4)
    avgthroughput = models.DecimalField(max_digits=10, decimal_places=8)
    summarized = models.CharField(max_length=1)
    class Meta:
        db_table = u'ftshourlysummary'
    
class FtsDailySummary(Model):
    timestamp = models.IntegerField(null=False, primary_key=True)
    void = models.IntegerField(null=False, primary_key=True)
    src_site_id = models.IntegerField(null=False, primary_key=True)
    dst_site_id = models.IntegerField(null=False, primary_key=True)
    aggregatedata = models.DecimalField(max_digits=10, decimal_places=4)
    avgthroughput = models.DecimalField(max_digits=10, decimal_places=8)
    summarized = models.CharField(max_length=1)
    class Meta:
        db_table = u'ftsdailysummary'

class FtsWeeklySummary(Model):
    timestamp = models.IntegerField(null=False, primary_key=True)
    void = models.IntegerField(null=False, primary_key=True)
    src_site_id = models.IntegerField(null=False, primary_key=True)
    dst_site_id = models.IntegerField(null=False, primary_key=True)
    aggregatedata = models.DecimalField(max_digits=10, decimal_places=4)
    avgthroughput = models.DecimalField(max_digits=10, decimal_places=8)
    summarized = models.CharField(max_length=1)
    class Meta:
        db_table = u'ftsweeklysummary'

class FtsMonthlySummary(Model):
    timestamp = models.IntegerField(null=False, primary_key=True)
    void = models.IntegerField(null=False, primary_key=True)
    src_site_id = models.IntegerField(null=False, primary_key=True)
    dst_site_id = models.IntegerField(null=False, primary_key=True)
    aggregatedata = models.DecimalField(max_digits=10, decimal_places=4)
    avgthroughput = models.DecimalField(max_digits=10, decimal_places=8)
    class Meta:
        db_table = u'ftsmonthlysummary'
