import calendar
from datetime import timedelta, datetime
from dateutil.relativedelta import relativedelta
import urllib2
import simplejson
import cStringIO as StringIO
import ho.pisa as pisa
from cgi import escape
import tempfile

from django.shortcuts import render_to_response
from django.views.decorators.cache import cache_page
from django.db.models import Avg, Sum, Count
from django.template.loader import get_template
from django.template import Context, RequestContext
from django.http import HttpResponse, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.template import Context
from django.core.mail import send_mail
from django.utils.datastructures import SortedDict

from mywlcg import settings
from mywlcg.filters.views import parse_filters_reports, authorized_dn
from mywlcg.base.views import path_prefix
from mywlcg.mdb.models import(
    ACE_D_SITE_AVAILABILITY, ACE_M_SITE_AVAILABILITY,
    AceAlgorithmProfileMap, Profile,
    T0T1SERVICETARGETS , SnapshotTopologyView
)


# build identifier list for the available reports

reports = ['EGI',
           'WLCG_Tier1_History',
           'WLCG_Tier1_Summary',
           'WLCG_Tier1_VO',
           'WLCG_Tier2',
           'WLCG_All_Sites']
rep = type('Reports', (object, ), {})
for i, name in enumerate(reports):
    setattr(rep, name, i)


# build reports filter options

report_options = {
    'WLCG': [
        ['All Sites', [
            ['All sites summary', rep.WLCG_All_Sites]
        ]],
        ['Tier 0+1', [
            ['Tier 0+1 sites summary', rep.WLCG_Tier1_Summary],
            ['Tier 0+1 sites complete', rep.WLCG_Tier1_VO],
            ['Tier 0+1 sites history', rep.WLCG_Tier1_History]
        ]],
        ['Tier 2', [
            ['Tier 2 sites summary', rep.WLCG_Tier2],
        ]],
    ],

    'EGI': [
        ['All Sites', [
            ['All sites summary', rep.EGI],
        ]],
    ],

    # reports that can display all configured profiles at once
    'ALL': [
        ['WLCG Tier 0+1', [
            ['Tier 0+1 sites summary', rep.WLCG_Tier1_Summary],
        ]]
    ]
}


def build_report_filters(egi_only=False):
    """
    builds a dictionary of the form {
        'vos': {'voname': profile_id},
        'report_options': {
            'infrast': [report group, [report 1, ...]]
        },
    } so the filters can be automatically set up by just specifying the
    profiles in the report settings.
    """
    vo_reports = {}
    for report in reports:
        infrast = 'WLCG'
        if 'EGI' in report:
            infrast = 'EGI'

        if infrast == 'WLCG' and egi_only:
            continue

        report_profiles = getattr(settings, "%s_PROFILES" % report.upper(), '').split()

        # add the reports in which each profile is defined to the vo
        for p in Profile.objects.filter(name__in=report_profiles):
            vo = p.atp_vo.voname
            if vo not in vo_reports:
                vo_reports[vo] = {}
            vo_reports[vo].update({
                p.id: infrast
            })

    if not egi_only:
        # add combined VO reports
        vo_reports['all'] = {'all': 'ALL'}

    return simplejson.dumps({
        'vos': vo_reports,
        'report_options': report_options,
    })


def render_to_pdf(template_src, context_dict):
    template = get_template(template_src)
    context = Context(context_dict)
    html  = template.render(context)
    result = StringIO.StringIO()
    pdf = pisa.pisaDocument(StringIO.StringIO(html.encode("ISO-8859-1")), result)
    if not pdf.err:
        return HttpResponse(result.getvalue(), mimetype='application/pdf')
    return HttpResponse('We had some errors<pre>%s</pre>' % escape(html))


def setdate(month, year, prev, history=0):
    # Dates
    if history:
        history = history-1
    fromdate = datetime(year, month, 1, 0, 0, 0) - relativedelta(months=prev) - relativedelta(months=history)
    todate = datetime(year, month, calendar.monthrange(year, month)[1]) - relativedelta(months=prev)

    return (fromdate, todate)


def index(request):
    return HttpResponseRedirect(reverse('mywlcg.reports.views.monthly'))


def monthly(request):
    results = {}
    results['reports'] = reports
    results['official'] = authorized_dn(request)

    egi_only = not results['official'] and 'myegi' in request.path
    results = parse_filters_reports(request, results, egi_only)

    report_filters = build_report_filters(egi_only)
    results['report_filters'] = report_filters
    results['reportlist'] = []

    results['csv'] = int(request.GET.get("csv") or 0)
    pdf = int(request.GET.get("pdf") or 0)

    if results['report'] == rep.EGI:
        results['reportlist'].append([rep.EGI, results['profile'], 'Name', 0])
        results['reportlist'].append([rep.EGI, results['profile'], 'Availability', 0])
        results['reportlist'].append([rep.EGI, results['profile'], 'Region', 0])

    elif results['report'] == rep.WLCG_Tier1_History:
        results['reportlist'].append([rep.WLCG_Tier1_History, results['profile'], '', 0, 6])

    elif results['report'] == rep.WLCG_Tier1_Summary:
        if results['profile'] == 'all':
            from mywlcg.filters.views import _get_configured_report_profiles
            for p in _get_configured_report_profiles('WLCG_Tier1_Summary'):
                results['reportlist'].append([rep.WLCG_Tier1_Summary, p.id, 'Availability', 0])
        else:
            results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Availability', 0])

    elif results['report'] == rep.WLCG_Tier1_VO:
        j=0
        results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Availability', 0, results['history']])
        while j<results['history']:
            results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Availability',j])
            results['reportlist'].append([rep.WLCG_Tier1_VO, results['profile'], 'Availability',j])
            results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Reliability',j])
            results['reportlist'].append([rep.WLCG_Tier1_VO, results['profile'], 'Reliability',j])
            j=j+1

    elif results['report'] == rep.WLCG_Tier2:
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Name', 0])
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Reliability', 0])
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Federation', 0])

    elif results['report'] == rep.WLCG_All_Sites:
        history = 0
        results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Availability', 0, history])
        results['reportlist'].append([rep.WLCG_Tier1_Summary, results['profile'], 'Reliability', 0, history])
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Name', 0])
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Reliability', 0])
        results['reportlist'].append([rep.WLCG_Tier2, results['profile'], 'Federation', 0])

    currdate = datetime.utcnow().date()
    prevdate=currdate - relativedelta(months=1)
    i=2008
    results['years']=[]
    while i<=currdate.year:
        results['years'].append(i)
        i=i+1
    results['month'] = int(request.GET.get("month") or prevdate.month)
    results['year'] = int(request.GET.get("year") or prevdate.year)
    fromdate, todate = setdate(results['month'],results['year'],0)
    export_filename = results['reports'][results['report']]

    if results['report'] in [rep.WLCG_Tier1_History, rep.WLCG_Tier1_Summary,
                             rep.WLCG_Tier1_VO, rep.WLCG_Tier2, rep.WLCG_All_Sites]:
        if results['profile'] != 'all':
            voname = Profile.objects.filter(
                id=results['profile']
            ).values_list('atp_vo__voname')[0][0].upper()
            export_filename += '_' + voname
    export_filename += '_' + todate.strftime('%B')[:3] + str(todate.year)

    if not results['report']:
        urlappend='egi'
    else:
        urlappend='wlcg'
    url=settings.REPORTS_HOST+str(todate.year)+'/'+str(todate.year)+str(todate.month).zfill(2)+'/'+urlappend+'/'

    def get_url_if_accessible(url):
        try:
            urllib2.urlopen(url)
        except urllib2.URLError:
            return None
        return url

    results['pdfurl'] = '%s%s.%s' % (url, export_filename, 'pdf')
    results['csvurl'] = '%s%s.%s' % (url, export_filename, 'csv')
    results['pdf_exists'] = get_url_if_accessible(results['pdfurl'])
    results['csv_exists'] = get_url_if_accessible(results['csvurl'])

    this_month = (results['month'] == currdate.month and results['year'] == currdate.year)

    if pdf:
        response = HttpResponse(mimetype='application/pdf')
        response['Content-Disposition'] = 'attachment; filename='+export_filename+'.pdf'
        if this_month or results['official']:
            layout=''
            data=[]
            hdata=[]
            for list in results['reportlist']:
                params = json(request,list)
                paramshead = heading(request,list)
                data.append((simplejson.loads(paramshead.content)['data'],simplejson.loads(params.content)['data']))
            if results['report'] in [rep.WLCG_Tier1_Summary, rep.WLCG_Tier1_VO, rep.WLCG_All_Sites]:
                layout = 'landscape'
            result = {}
            path_prefix(request,result)
            result['PATH'] = 'http://' + request.META.get('HTTP_HOST') + result['PATH']
            response.write(render_to_pdf('reports_pdf.html',
                {'data':data, 'layout': layout,
                'PATH': result['PATH'], 'official': results['official']},
            ))
        else:
            req = urllib2.Request(results['pdfurl'])
            opener = urllib2.build_opener()
            f = opener.open(req)
            response.write(f.read())
        return response
    elif results['csv']:
        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename='+export_filename+'.csv'
        if this_month or results['official']:
            FILE = tempfile.NamedTemporaryFile('w+')
            for list in results['reportlist']:
                heading(request,[FILE]+list)
                json(request,[FILE]+list)
            FILE.flush()
            FILE.seek(0)
            response.write(FILE.read())
            FILE.close()
        else:
            req = urllib2.Request(results['csvurl'])
            opener = urllib2.build_opener()
            f = opener.open(req)
            response.write(f.read())
        return response

    else:
        if this_month or results['official']:
            results['generate'] = 1
        else:
            results['generate'] = 0
            results['iframeheight']=1315
            if results['report'] in [2, 3]:
                results['iframeheight'] = 665
        results['tab'] = 'monthly'
        return render_to_response('reports_monthly.html', results)


#@cache_page(600)
def heading(request, settings=None):
    if settings is None:
        settings = []
    profile = request.GET.get("profile")
    report = int(request.GET.get("report") or 2)
    official = authorized_dn(request)
    display = request.GET.get("type")
    prev = int(request.GET.get("prev") or 0)
    history = int(request.GET.get("history") or 0)
    csv = int(request.GET.get("csv") or 0)
    counter = int(request.GET.get("counter") or 1)
    month = int(request.GET.get("month"))
    year = int(request.GET.get("year"))
    pdf = 0

    if len(settings) == 4:
        report, profile, display, prev = settings
        history = 0
        pdf = 1

    elif len(settings) == 5:
        if csv:
            FILE, report, profile, display, prev = settings
            history = 0
        else:
            report, profile, display, prev, history = settings
            pdf = 1

    elif len(settings) == 6:
        FILE, report, profile, display, prev, history = settings

    fromdate, todate = setdate(month, year, prev, history)
    voname = Profile.objects.filter(id=profile).values_list('atp_vo__voname')[0][0]
    profilename = Profile.objects.get(id=profile).name

    if report == rep.WLCG_Tier2:
        summarytype = 'Federation'
        rtype = 'Tier-2'
        width = '450'
        ccwidth = '30%'
        ccoding = ("<td>N/A</td><td style='background-color:red'><30%</td>"
                   "<td style='background-color:orange'><60%</td>"
                   "<td style='background-color:gold'><90%</td>"
                   "<td style='background-color:yellowgreen'>>=90%</td>")
    else:
        summarytype = 'Region'
        rtype = 'EGI'
        width = '350'
        ccwidth = '40%'
        ccoding = ("<td>N/A</td><td style='background-color:red'>&lt;Target</td>"
                   "<td style='background-color:yellowgreen'>>=Target</td>")

    results = ['','','','','']
    targetts = T0T1SERVICETARGETS.objects.filter(
        effectivefromts__lte=fromdate
    ).order_by('-effectivefromts').values_list('effectivefromts')
    data = T0T1SERVICETARGETS.objects.filter(
        effectivefromts__gte=targetts[0][0]
    ).order_by('-effectivefromts')

    size = '2'
    if pdf:
        size = '4'

    algo = AceAlgorithmProfileMap.objects.filter(profile=profile).values_list('algorithm__site_algo')
    if len(algo):
        algo = algo[0][0]
    else:
        algo = "'And' of all Service Flavours"
    results[4] = 'Availability Algorithm: ' + algo

    date = results[2] = ''
    if not csv:
        date=results[2] = '<font color=blue>'
    if history:
        results[2] = results[2] + fromdate.strftime('%B') + ' ' + str(fromdate.year) + ' - '
    results[2] = results[2] + todate.strftime('%B') + ' ' + str(todate.year)
    date = date + todate.strftime('%B') + ' ' + str(todate.year)
    if not csv:
        results[2] = results[2] + '</font>'
        date = date + '</font>'
        results[4] = results[4] + '</a></font><br/>'

    if type == 'Availability':
        data = data.values_list('availtargetpersite','availtargetbestsites')[0]
    else:
        data = data.values_list('reltargetpersite','reltargetbestsites')[0]

    # display profile name along with VO
    # results[1] = profilename+' Profile for '+voname+' VO'
    results[1] = voname.upper()
    if report in [rep.EGI, rep.WLCG_Tier2]:
        results[0] = rtype + ' Availability and Reliability Report'
        if display in ('Federation', 'Region'):
            results[3] = display + ' Details'
        else:
            results[3] = summarytype + ' Summary - Sorted by ' + display
        border = ''
        space = ''
        if pdf:
            border = 'border=1 bordercolor=white'
            space = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
        if not csv:
            results[3] = results[3] + (
                "<table width=" + width + "px " + border + " style='padding-top:2px;margin-bottom:2px'>"
                "<tr><td width=" + ccwidth + ">" + space + "Colour coding :</td>" + ccoding + '</tr></table>'
            )
        if report == rep.EGI:
            results[3] = results[3] + 'EGI SLA Availability Target is 70% and Reliability Target is 75%'

    elif report == rep.WLCG_Tier1_History:
        results[3] = results[2]
        if not csv:
            results[3] = results[3] + (
                '<br/><font color=blue size=' + size + '>Target ' + display + ' for each site is ' + str(data[0]*100) + '%</font>'
                '<br/>Colors: <font color=green size=' + size + '>Green > Target</font>'
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font color=darkorange>Orange > 90% of Target</font>'
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font color=red>Red < 90% of Target</font>'
            )
        results[2] = date
        results[0] = 'WLCG Sites Reliability'

    elif report == rep.WLCG_Tier1_Summary:
        offstr = ""
        if not official and pdf:
            offstr = " (Unofficial Report)"
        results[0] = display + ' of WLCG Tier-0 + Tier-1 Sites' + offstr
        if not csv:
            results[3] = '<font color=blue size='+size+'>'
        results[3]= results[3] + (
            'Target ' + display + ' for each site is ' + str(data[0]*100) + '%.'
            'Target for 8 best sites is ' + str(data[1]*100) + '%'
        )
        if not csv:
            results[3] = results[3] + '</font>'

    elif report == rep.WLCG_Tier1_VO:
        results[0] = 'Service ' + display + ' Data for WLCG Tier-0 + Tier-1 Sites'
        if not csv:
            results[3] = '<font color=blue size=' + size + '>'
        results[3] = results[3] + 'Target ' + display + ' for each site is ' + str(data[0]*100) + '%'
        if not csv:
            results[3] = results[3] + (
                '</font><br/>Colors: <font color=green>Green > Target</font>'
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font color=darkorange>Orange > 90% of Target</font>'
                '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<font color=red>Red < 90% of Target</font>'
            )

    result = {}
    path_prefix(request,result)
    result['PATH'] = 'http://' + request.META.get('HTTP_HOST') + result['PATH']
    span = 0
    if report not in [rep.WLCG_Tier1_Summary, rep.WLCG_Tier1_VO] and pdf:
        span = 1

    if not csv:
        callback='reports_heading'
        return render_to_response(callback, {
            "results": results,
            "PATH": result['PATH'],
            "counter": counter,
            "report": report,
            "pdf": pdf,
            "span": span,
        })
    else:
        a = FILE.name
        FILE.write('\n\n\n')
        for result in results:
            FILE.write('\n,,'+result+',,,')
        FILE.write('\n\n\n')


def getcolor(csv,rel,sitedata,targetpersite,count=1,sumunkn=0,count1=1):
    if count1==1 and count!=0:
        count1=count
    if rel:
        avail=sitedata
        avail1=int(round(avail*100))
    else:
        if not len(sitedata) or sitedata[0][1]==None:
            unkn=avail=-1
        else:
            unkn=sitedata[0][1]/count1
            if sumunkn:
                unkn=sumunkn/count1
            unkn1=int(round(unkn*100))
            if count-sitedata[0][1]<0.005:
                avail=-1
            else:
                avail=sitedata[0][0]/(count-sitedata[0][1])
                avail1=int(round(avail*100))
    if avail==-1:
        avail1='N/A'
    if rel in [0,1]:
        targetpersite=targetpersite*100
        if avail1=='N/A':
            color='black'
        elif avail1>=targetpersite:
            color='green'
        elif avail1>targetpersite*0.90:
            color='darkorange'
        else:
            color='red'
    if rel==1:
        if csv:
            return avail1
        return "<font color="+color+">"+str(avail1)+"</font>"
    elif rel==2:
        if avail1=='N/A':
            color='white'
        elif avail1>=targetpersite[0]:
            color='yellowgreen'
        elif avail1>=targetpersite[1]:
            color='gold'
        elif avail1>=targetpersite[2]:
            color='orange'
        else:
            color='red'
        if avail1!='N/A':
            avail1=str(avail1)+' %'
        if csv:
            return str(avail1)
        return "<td align=center style='background-color:"+color+"'>"+avail1+"</td>"
    else:
        if unkn==-1:
            unkn1='N/A'
        if csv:
            return (avail,str(avail1),unkn,str(unkn1))
        return (avail,"<font color="+color+">"+str(avail1)+"</font>",unkn,"<font color=grey>"+str(unkn1)+"</font>")


#@cache_page(600)
def json(request, list=[]):
    if not list:
        profile = int(request.GET.get("profile"))
    report = int(request.GET.get("report") or 2)
    type = request.GET.get("type")
    prev = int(request.GET.get("prev") or 0)
    history = int(request.GET.get("history") or 0)
    counter = int(request.GET.get("counter") or 1)
    month = int(request.GET.get("month"))
    year = int(request.GET.get("year"))
    csv = int(request.GET.get("csv") or 0)
    pdf = 0
    size = 2
    pdf = 0
    if len(list)==4:
        report,profile,type,prev=list
        history=0
        size=4
        pdf=1
    elif len(list)==5:
        if csv:
            FILE,report,profile,type,prev=list
            history=0
        else:
            report,profile,type,prev,history=list
            pdf=1
            size=4
    elif len(list)==6:
        FILE,report,profile,type,prev,history=list
    fromdate, todate = setdate(month, year, prev, history)
    voname = Profile.objects.filter(id=profile).values_list('atp_vo__voname')[0][0]
    profilename = Profile.objects.get(id=profile).name

    if report in [rep.WLCG_Tier1_History, rep.WLCG_Tier1_Summary, rep.WLCG_Tier1_VO]:
        if voname == 'ops':
            field = 'fedname'
        else:
            field = 'logsitename'

        latest_snap = SnapshotTopologyView.objects.filter(
            snapshotts__lt=todate
        ).values_list('snapshotts', flat=True).order_by('-snapshotts')
        snap_constraint = {}
        if len(latest_snap):
            snap_constraint = {'snapshotts': latest_snap[0]}

        groups = SnapshotTopologyView.objects.filter(
            profilename=profilename,
            tier__in=('Tier-0', 'Tier-1'),
            **snap_constraint
        ).values_list(
            'loggroupingid', 'logsitename', field
        ).order_by('tier', field).distinct()

        targetts = T0T1SERVICETARGETS.objects.filter(
            effectivefromts__lte=fromdate
        ).order_by('-effectivefromts').values_list('effectivefromts')
        targetpersite = T0T1SERVICETARGETS.objects.filter(
            effectivefromts__gte=targetts[0][0]
        ).values_list('availtargetpersite','reltargetpersite').order_by('-effectivefromts')[0]

        gids=groups.values_list('loggroupingid')
        Ddata=ACE_D_SITE_AVAILABILITY.objects.filter(
                                 PROFILE=profile,
                                 TIMESTAMP__range=(fromdate, todate))
        Mdata=ACE_M_SITE_AVAILABILITY.objects.filter(
                                 PROFILE=profile,
                                 TIMESTAMP__range=(fromdate, todate),
                                 GROUPING__in=gids)

    if report == rep.WLCG_Tier1_History:
        results={'results':[],'pdf':pdf,'size':size}
        callback='reports_Tier1_history_csv'
        if not csv:
            callback='reports_Tier1_history_json'

        # Reliability for 8 best sites and average per month

        caldata=[('Month','Reliability')]
        ts=Mdata.order_by('TIMESTAMP').values_list('TIMESTAMP',flat=True).distinct()
        for date in ts:
            data=Mdata.filter(TIMESTAMP=date).values_list('AUP','ADOWN')
            rellist=[]
            count=0
            for rdata in data:
                if rdata[0]+rdata[1]>0.005:
                    rellist.append((rdata[0]/(rdata[0]+rdata[1]),count))
                else:
                    rellist.append((-1,count))
                count=count+1
            rellist.sort(key=lambda x: float(x[0]))
            rellist=rellist[-8:]
            sumup=0
            sumdown=0
            count=0
            for index in rellist:
                if data[index[1]][0]+data[index[1]][1]>0.005:
                    sumup=sumup+data[index[1]][0]
                    sumdown=sumdown+data[index[1]][1]
                    count=count+1
            rel=-1
            if sumup+sumdown>0.005:
                rel=sumup/(sumup+sumdown)
            caldata.append((date.strftime('%B')[:3]+' '+str(date.year),getcolor(csv,1,rel,targetpersite[1])))
        results['results'].append({'heading':'Average of 8 best sites (not always the same 8)','data':caldata})

        data=Mdata.values('TIMESTAMP').annotate(Sum('AUP'),Sum('ADOWN'))\
                         .order_by('TIMESTAMP').values_list('TIMESTAMP','AUP__sum','ADOWN__sum')
        caldata=[('Month','Reliability')]
        for ldata in data:
            rel=-1
            if ldata[1]+ldata[2]>0.005:
                rel=ldata[1]/(ldata[1]+ldata[2])
            caldata.append((ldata[0].strftime('%B')[:3]+' '+str(ldata[0].year),getcolor(csv,1,rel,targetpersite[1])))
        results['results'].append({
                         'heading':'Average of ALL Tier-0 and Tier-1 sites',
                         'data':caldata
                 })

        # Reliability summary per federation
        # monthly reliabilities are stored as a function of their hspec, and then
        # reduced by the total hspec for the sites in the same federation

        caldata = []  # final data to be rendered
        fed_reliabilities = SortedDict()  # modified values for each month per federation
        fed_hspec = {}  # total hspec capacity for each federation, month

        for group in groups:
            gid, gname, fedname = group
            rdata = {}  # reliability values per federation
            for i, month_ts in enumerate(ts):
                rel = Mdata.filter(GROUPING=gid, TIMESTAMP=month_ts).values_list('RELIABILITY')
                if len(rel):
                    rel = rel[0][0]
                else:
                    rel = -1

                # get federation and hspec for the precise month
                siteinfo = SnapshotTopologyView.objects.filter(
                    loggroupingid=gid, profilename=profilename, tier__in=('Tier-0', 'Tier-1'),
                    # snapshot may be taken the 1st, so we check for a ts earlier than next month
                    snapshotts__lt=month_ts + timedelta(days=31)
                ).order_by('-snapshotts').values_list('fedname', 'hspec06')

                if len(siteinfo):
                    fedname, hspec = siteinfo[0]
                if rel == -1 or hspec == -1:
                    hspec = 1.0

                # store reliability value for this month and federation
                if fedname not in rdata:
                    rdata[fedname] = [-1] * len(ts)
                rdata[fedname][i] = rel * hspec
                fed_hspec[(fedname, i)] = fed_hspec.get((fedname, i), 0.0) + hspec

            # aggregate reliability of the sites from the same federations
            for fed in rdata:
                if fed not in fed_reliabilities:
                    fed_reliabilities[fed] = rdata[fed]
                else:
                    for i, r in enumerate(fed_reliabilities[fed]):
                        fed_reliabilities[fed][i] += rdata[fed][i]

        # normalize reliabilities for hspec total capacity
        for fed in fed_reliabilities:
            for (i, rel) in enumerate(fed_reliabilities[fed]):
                if (fed, i) in fed_hspec:
                    cap_factor = max(fed_hspec[(fed, i)], 1.0)
                    fed_reliabilities[fed][i] /= cap_factor

        fmt_date = lambda x: "%s %s" % (x.strftime('%B')[:3], str(x.year))
        caldata = [['Site'] + map(fmt_date, ts)]

        for fed, rels in fed_reliabilities.iteritems():
            rels_html = map(lambda r: getcolor(csv, 1, r, targetpersite[1]), rels)
            caldata.append([fed] + rels_html)

        caldata.append(['Target'] + [int(targetpersite[1] * 100)] * len(ts))
        results['results'].append({
            'heading': 'Detailed Monthly Site Reliability (%s tests)' % voname,
            'data': caldata,
        })

        if csv:
            for result in results['results']:
                FILE.write('\n\n'+result['heading']+',')
                for rdata in result['data']:
                    FILE.write('\n')
                    for ldata in rdata:
                        FILE.write(str(ldata)+',')

    elif report == rep.WLCG_Tier1_Summary:
        results = {
            'profile': profilename,
            'start_time': str(fromdate.year) + '-' + str(fromdate.month) + '-01T00:00:00Z',
            'end_time': str(todate.year) + '-' + str(todate.month) + '-' + str(calendar.monthrange(todate.year, todate.month)[1]) + 'T00:00:00Z',
            'type': type,
            'groupdata': [],
        }
        callback='reports_vowise_graphs_json'
        for group in groups:
            sitedata=Ddata.filter(GROUPING=group[0]).values('GROUPING')\
                            .annotate(Avg('AUNKNOWN'),Sum('AUNKNOWN'),Sum('AMAINTENANCE'),Sum('AUP'),Count('GROUPING'))\
                            .values_list('AUNKNOWN__avg','AUNKNOWN__sum','AMAINTENANCE__sum','GROUPING__count','AUP__sum')
            temp1=temp2=0
            if len(sitedata):
                temp1=sitedata[0][3]-sitedata[0][1]
                temp2=sitedata[0][3]-(sitedata[0][1]+sitedata[0][2])
                unkn=str(int(round(sitedata[0][0]*100)))+'%'
            else:
                unkn='N/A'
            if temp1> 0.005:
                avail=str(int(round(sitedata[0][4]*100/temp1)))+'%'
            else:
                avail='N/A'
            if temp2>0.005:
                reli=str(int(round(sitedata[0][4]*100/temp2)))+'%'
            else:
                reli='N/A'
            results['groupdata'].append({'groupname': group[1],
                                         'sitename': group[2],
                                         'availability':avail,
                                         'reliability':reli,
                                         'unknown':unkn,
                            })
            results['pdf']=pdf
        if csv:
            if type=='Availability':
                FILE.write('\nSite,,Availability,Unknown')
            else:
                FILE.write('\nSite,,Reliability')
            for result in results['groupdata']:
                if type=='Availability':
                    FILE.write('\n'+result['groupname']+',,'+result['availability']+','+result['unknown'])
                else:
                    FILE.write('\n'+result['groupname']+',,'+result['reliability'])

    elif report == rep.WLCG_Tier1_VO:
        results={'groupname':[],'data':[],'type': type,'pdf':pdf}

        callback='reports_vowise_table_json'
        start=fromdate
        i=0
        csumavail= [0] * 31
        cactualsumunkn= [0] * 31
        csumunkn= [0] * 31
        m1=[0]*31
        if type=='Availability':
            siteavail=Mdata.aggregate(Sum('AUP'),Sum('AUNKNOWN'),Count('GROUPING'))
            siteavail8=Mdata.values_list('AUP','AUNKNOWN')
        else:
            siteavail=Mdata.aggregate(Sum('AUP'),Sum('ADOWN'))
            siteavail8=Mdata.values_list('AUP','ADOWN')
        siteavail8list=[]
        i=0
        for no in siteavail8:
            if type=='Availability':
                num1=num2=1
            else:
                num1=no[0]
                num2=-1
            if num1-num2*no[1]>0.005:
                siteavail8list.append((no[0]/(num1-num2*no[1]),i))
            else:
                siteavail8list.append((0.0,i))
            i=i+1
        siteavail8list.sort(key=lambda x: float(x[0]))
        siteavail8list=siteavail8list[-8:]
        sumavail8=0
        sumunkn8=0
        k=0
        count=0
        for index in siteavail8list:
            num1=siteavail8[index[1]][0]
            num2=-1
            if num1-num2*siteavail8[index[1]][1]>0.005:
                sumunkn8=sumunkn8+siteavail8[index[1]][1]
                sumavail8=sumavail8+siteavail8[index[1]][0]
                count=count+1
        if type=='Availability':
            avgsiteavail,avgsiteavail1,avgsiteunkn,avgsiteunkn1=getcolor(csv,0,[(siteavail['AUP__sum'],siteavail['AUNKNOWN__sum'])]\
                                                                   ,targetpersite[0],siteavail['GROUPING__count'])
            avgsiteavail8,avgsiteavail18,avgsiteunkn8,avgsiteunkn18=getcolor(csv,0,[(sumavail8,sumunkn8)],targetpersite[0],count)
        else:
            avgsiteavail,avgsiteavail1,avgsiteunkn,avgsiteunkn1=getcolor(csv,0,[(siteavail['AUP__sum'],-1*siteavail['ADOWN__sum'])]\
                                                                   ,targetpersite[0],siteavail['AUP__sum'])
            avgsiteavail8,avgsiteavail18,avgsiteunkn8,avgsiteunkn18=getcolor(csv,0,[(sumavail8,-1*sumunkn8)],targetpersite[0],sumavail8)
        i=[0] * 31
        while start <= todate:
            data=[]
            j=0
            m=k=n=0

            rsumavail=0
            ractualsumunkn=rsumunkn=0
            for group in groups:
                if start==fromdate:
                    results['groupname'].append(group[2])
                sitedata=Ddata.filter(GROUPING=group[0],TIMESTAMP=start)
                if type=='Availability':
                    sitedata=sitedata1=sitedata.values_list('AUP','AUNKNOWN')
                    countdata=1
                else:
                    sitedata=sitedata.values_list('AUP','ADOWN')
                    sitedata1=[(-1,-1)]
                    if len(sitedata):
                        sitedata1=[(sitedata[0][0],-1*sitedata[0][1])]
                    countdata=sitedata1[0][0]
                avail,avail1,unkn,unkn1=getcolor(csv,0,sitedata1,targetpersite[0],countdata)
                if avail>=0:
                    rsumavail=rsumavail+sitedata[0][0]
                    csumavail[j]=csumavail[j]+sitedata[0][0]
                    k=k+1
                    rsumunkn=rsumunkn+sitedata[0][1]
                    csumunkn[j]=csumunkn[j]+sitedata[0][1]
                    i[j]=i[j]+1
                    if type=='Availability':
                        m=m+1
                        m1[j]=m1[j]+1
                        ractualsumunkn=ractualsumunkn+sitedata[0][1]
                        cactualsumunkn[j]=cactualsumunkn[j]+sitedata[0][1]
                if unkn>0 and avail<0 and type=='Availability':
                    ractualsumunkn=ractualsumunkn+sitedata[0][1]
                    cactualsumunkn[j]=cactualsumunkn[j]+sitedata[0][1]
                    m=m+1
                    m1[j]=m1[j]+1
                j=j+1
                if type=='Availability':
                    data.append((avail1,unkn1))
                else:
                    data.append(avail1)
            if k>0:
                if type=='Availability':
                    if m==0:
                        m=k
                    rsumavail,rsumavail1,rsumunkn,rsumunkn1=getcolor(csv,0,[(rsumavail,rsumunkn)],targetpersite[0],k,ractualsumunkn,m)
                    data.append((rsumavail1,rsumunkn1))
                else:
                    rsumavail,rsumavail1,rsumunkn,rsumunkn1=getcolor(csv,0,[(rsumavail,-1*rsumunkn)],targetpersite[0],rsumavail)
                    data.append(rsumavail1)
                results['data'].append([str(start.strftime("%d/%m/%Y").lstrip("0")), data])
            start=start+timedelta(days=1)
        k=0
        data=[]
        for group in groups:
            if type=='Availability':
                if m1[k]==0:
                    m1[k]=i[k] or 1
                csa,csumavail1,csu,csumunkn1=getcolor(csv,0,[(csumavail[k],csumunkn[k])],targetpersite[0],i[k],cactualsumunkn[k],m1[k])
                data.append((csumavail1,csumunkn1))
            else:
                csa,csumavail1,csu,csumunkn1=getcolor(csv,0,[(csumavail[k],-1*csumunkn[k])],targetpersite[0],csumavail[k])
                data.append(csumavail1)
            k=k+1
        if type=='Availability':
            data.append((avgsiteavail1,avgsiteunkn1))
        else:
            data.append(avgsiteavail1)
        results['avgsiteavail']=avgsiteavail1
        results['avg8bestsiteavail']=avgsiteavail18

        results['data'].append(['Average',data])
        results['groupname'].append('Average')
        if csv:
            FILE.write('\n,,')
            for result in results['groupname']:
                if type=='Availability':
                    FILE.write(result+',,,')
                else:
                    FILE.write(result+',,')
            FILE.write('\n,,')
            for result in results['groupname']:
                if type=='Availability':
                    FILE.write('Availability,Unknown,,')
                else:
                    FILE.write('Reliability,,')
            for data in results['data']:
                FILE.write('\n')
                scount=0
                for sdata in data:
                    if scount==0:
                        scount=1
                        FILE.write(sdata+',,')
                    else:
                        for adata in sdata:
                            if type=='Availability':
                                FILE.write(adata[0]+','+adata[1]+',,')
                            else:
                                FILE.write(adata+',,')
            FILE.write('\n\nAverage '+type+' for all sites: '+str(results['avgsiteavail'])+' %')
            FILE.write('\nAverage '+type+' for 8 best sites: '+str(results['avg8bestsiteavail'])+' %')

    elif report in [rep.EGI, rep.WLCG_Tier2]:
        if type in['Availability', 'Reliability', 'Name']:
            callback = 'reports_table_sorted_json'
            num = 1
            mstr = '-'
        else:
            callback = 'reports_table_detailed_json'
            num = 4
            mstr = ''
        results = {'report':report, 'month':[], 'pdf':pdf}
        count = 0
        datalist = []

        # get latest applying snapshot timestamp
        snap_constraint = {}
        ts = SnapshotTopologyView.objects.filter(
                 # use next day as top date to include last day's topology
                 snapshotts__range=(fromdate, todate + timedelta(days=1))
             ).values_list('snapshotts', flat=True).order_by('-snapshotts')
        if len(ts):
            topology_timestamp = ts[0]
            snap_constraint = {'snapshotts': topology_timestamp}
        snapshot = SnapshotTopologyView.objects.filter(profilename=profilename, **snap_constraint)

        if report:
            snapshot = snapshot.filter(
                           tier__in=['Tier2', 'Tier 2', 'Tier-2'],
                           fedacname__isnull=False
                       ).order_by('fedacname')
            fednames = snapshot.values_list('fedacname').distinct().order_by(mstr + 'fedacname')
            targetpersiterel = targetpersiteavail=[90,60,30]
        else:
            snapshot = snapshot.filter(
                           certifstatus='Y', infrasttype='Production'
                       ).exclude(
                           region='OpenScienceGrid'
                       )
            fednames = snapshot.values_list('region').distinct().order_by(mstr + 'region')
            targetpersiteavail = [70, 110, 110]
            targetpersiterel = [75, 110, 110]

        glist = {}
        while count<num:
            data=[]
            fcount=0
            for fedname in fednames:
                if count==0:
                    glist[fedname[0]]=[]
                if num!=1 and len(datalist)==fcount:
                    if not csv:
                        datalist.append(['<tr><td colspan=11><hr></td></tr><tr><td colspan=2><b>'+fedname[0]+'</b></td><td colspan=9></td></tr>'])
                    else:
                        datalist.append([fedname[0]])

                snap_constraint = {}
                ts = SnapshotTopologyView.objects.filter(
                    snapshotts__range=(fromdate, todate + timedelta(days=1))
                ).values_list('snapshotts', flat=True).order_by('-snapshotts')
                if len(ts):
                    snap_constraint = {'snapshotts': ts[0]}
                if not report:
                    snapshot = SnapshotTopologyView.objects.filter(
                        profilename=profilename, **snap_constraint
                    )
                    groups = snapshot.filter(region=fedname[0],certifstatus='Y',infrasttype='Production')
                else:
                    groups=snapshot.filter(fedacname=fedname[0])
                gnames=groups.values_list('logsitename',flat=True).order_by('logsitename').distinct()
                groups=groups.values_list('loggroupingid','logsitename','hspec06','phycpucount','logcpucount')\
                                  .order_by('logsitename').distinct()
                escapelist=[]
                if count!=0:
                    for gdata in glist[fedname[0]]:
                        if gdata not in gnames:
                            avail=getcolor(csv,2,-1,targetpersiteavail)
                            if count==3 and not csv:
                                avail=avail+'</tr>'
                            escapelist.append(glist[fedname[0]].index(gdata)+1)
                            datalist[fcount][glist[fedname[0]].index(gdata)+1].append(avail)
                list=[]
                sumavail=sumrel=sumica=sumicr=0
                gcount=1
                for group in groups:
                    if count==0:
                        glist[fedname[0]].append(group[1])
                    if gcount in escapelist:
                        gcount=gcount+1
                    if group[1] in glist[fedname[0]]:
                        Mdata = ACE_M_SITE_AVAILABILITY.objects.filter(
                                      PROFILE=profile,
                                      TIMESTAMP__range=(fromdate,todate+timedelta(seconds=86399)),
                                      GROUPING=group[0]
                        ).values_list('AVAILABILITY','RELIABILITY','AUNKNOWN')
                        if len(Mdata):
                            avail=Mdata[0][0]
                            rel=Mdata[0][1]
                            unkn=int(round(Mdata[0][2]*100))
                        else:
                            avail=-1
                            rel=-1
                            unkn=-1
                        if num==1:
                            ica=group[2]
                            icr=group[2]
                            if avail==-1:
                                avail=0
                                ica=0
                            elif ica==0 or ica==None:
                                ica=1
                            if rel==-1:
                                rel=0
                                icr=0
                            elif icr==0 or icr==None:
                                icr=1
                            sumavail=sumavail+(avail*ica)
                            sumrel=sumrel+(rel*icr)
                            sumica=sumica+ica
                            sumicr=sumicr+icr
                        else:
                            ica=group[2]
                            if ica==0 or ica==None or ica==-1:
                                ica='N/A'
                            phy=group[3]
                            if phy==None or phy==-1:
                                phy='N/A'
                            log=group[4]
                            if log==None or log==-1:
                                log='N/A'
                            if unkn==None or unkn==-1:
                                unkn='N/A'
                            else:
                                unkn=str(unkn)+' %'
                            x=gnames[len(gnames)-1]
                            if count==0 and (report or avail>=0) and len(datalist[fcount])==gcount:
                                if not csv:
                                    datalist[fcount]=datalist[fcount]+[[
                                                                 '<tr><td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'\
                                                                 +'&nbsp;&nbsp;&nbsp;&nbsp;'+group[1]+'</td>',
                                                                 '<td align=center>'+str(phy)+'</td>',
                                                                 '<td align=center>'+str(log)+'</td>',
                                                                 '<td align=center>'+str(ica)+'</td>',
                                                                 getcolor(csv,2,avail,targetpersiteavail),
                                                                 getcolor(csv,2,rel,targetpersiterel),
                                                                 "<td align=center style='background-color:lightgrey'>"+unkn+'</td>'
                                                         ]]
                                else:
                                    datalist[fcount]=datalist[fcount]+[[
                                                                group[1],
                                                                str(phy),
                                                                str(log),
                                                                str(ica),
                                                                getcolor(csv,2,avail,targetpersiteavail),
                                                                getcolor(csv,2,rel,targetpersiterel),
                                                                unkn,
                                                         ]]
                            elif count==0 and avail<0:
                                glist[fedname[0]].remove(group[1])
                                gcount=gcount-1
                            else:
                                avail=getcolor(csv,2,avail,targetpersiteavail)
                                if count==3 and not csv:
                                    avail=avail+'</tr>'
                                datalist[fcount][gcount].append(avail)
                        gcount=gcount+1
                if num==1:
                    if sumica!=0:
                        availability=sumavail/sumica
                    else:
                        availability=-1
                    if sumicr!=0:
                        reliability=sumrel/sumicr
                    else:
                        reliability=-1
                    if not csv:
                        data.append(('<td width=25%>'+fedname[0]+'</td>',availability,reliability))
                    else:
                        data.append((fedname[0],availability,reliability))
                fcount=fcount+1

            if num==1:
                if type in ['Availability','Reliability']:
                    data.sort(key=lambda x: float(x[1]))
                data.reverse()
                data1=map(lambda x:[x[0],getcolor(csv,2,x[1],targetpersiteavail),getcolor(csv,2,x[2],targetpersiterel)], data)
                halflen=len(data1)/2+len(data1)%2
                number=0
                order=[]
                if not csv:
                    while number<halflen:
                        order=order+[number]
                        if number+halflen<len(data1):
                            order=order+[number+halflen]
                        number=number+1
                    results['data']=[data1[i] for i in order]
                else:
                    results['data']=data1
            if count==0:
                month=3
            else:
                month=-1
            fromdate=fromdate - relativedelta(months=month)
            todate=todate - relativedelta(months=month)
            results['month'].append(fromdate.strftime('%B')[:3]+' '+str(fromdate.year))
            count=count+1
        if num!=1:
            results['data']=datalist
        if csv:
            strname='Region'
            if report:
                strname='Federation'
            if num==1:
                FILE.write('\n'+strname+',,,Availability,Reliability')
                for data in results['data']:
                    FILE.write('\n')
                    count=0
                    for sdata in data:
                        if count==0:
                            count=1
                            FILE.write(sdata+',,,')
                        else:
                            FILE.write(sdata+',')
            else:
                if report:
                    FILE.write('\n,,,,,,,,,,,,Availability History,')
                    FILE.write('\n'+strname+',,,Site,,,Phy-CPU,Log-CPU,HS06,Availability,Reliability,Unknown,')
                FILE.write(results['month'][0]+','+results['month'][1]+','+results['month'][2])
                for data in results['data']:
                    count=0
                    for feddata in data:
                        if count==0:
                            count=1
                            FILE.write('\n'+feddata+',')
                        else:
                            gcount=0
                            for gdata in feddata:
                                if gcount==0:
                                    gcount=1
                                    FILE.write('\n,,,'+gdata+',,,')
                                else:
                                    FILE.write(gdata+',')
    if not csv:
        path_prefix(request,results)
        results['PATH'] = 'http://' + request.META.get('HTTP_HOST') + results['PATH']
        results['counter']=counter
        return render_to_response( callback, results)

def email(request,report,month,year):
    month=int(month)
    send=int(request.GET.get("send") or 0)
    final=int(request.GET.get("final") or 0)
    mstr=calendar.month_abbr[month]
    dstr=(datetime.utcnow().date()+timedelta(days=10))
    if final:
        cstr='final '
    else:
        cstr='draft '
    if report=='EGI':
        urlappend='egi'
        eappend=''
        cstr=cstr+report
        hstr=report
    else:
        urlappend='wlcg'
        eappend='WLCG_'
        hstr='WLCG '+report[:4]+'-'+report[4]
        cstr=cstr+report[:4]+' '+report[4]
    next='<br/>'
    if send:
        next='\n'
    content='Please find below the '+cstr+' availability and reliability reports for '+mstr+' '+year+next
    for rname in reports:
        pos=rname.find(report)
        if pos>-1:
            for profile in eval('settings.'+rname.upper()+'_PROFILES').split( ):
                export_filename=rname
                if reports.index(rname)==3:
                    export_filename=export_filename+'_'\
                               +Profile.objects.filter(name=profile).values_list('atp_vo__voname')[0][0].upper()
                export_filename=export_filename+'_'+mstr+str(year)
                url=settings.REPORTS_HOST+str(year)+'/'+str(year)+str(month).zfill(2)+'/'\
                         +urlappend+'/'+export_filename
                content+= next+url+'.pdf'
    if not final:
        content+=next+next+'Please verify your data and send comments via GGUS tickets by '+str(dstr.day)\
                         +' '+calendar.month_abbr[dstr.month]+' '+str(dstr.year)
    content+=next+next+'Kind regards,'+next+'SAM Team'
    email=eval('settings.'+eappend+report.upper()+'_EMAILS').split( )
    heading=hstr+' availability and reliability - '+mstr+' '+str(year)
    from_addr='sam-support@cern.ch'
    if send:
        try:
            send_mail(heading, content, from_addr,
                  email, fail_silently=False)
        except BadHeaderError:
            return render_to_response('reports_email.html',{'text':'Invalid header found.'})
        return render_to_response('reports_message.html',{'text':'Mail Send Successfully.'})
    else:
        return render_to_response('reports_email.html',
                                        {'from':from_addr,
                                         'to': email,
                                         'heading': heading,
                                         'content': content,
                                         'final': final
                                })
