from os import path as os_path
from ConfigParser import RawConfigParser, NoSectionError, NoOptionError
from django.core.exceptions import ImproperlyConfigured

# Django settings for MyWLCG project
PROJECT_NAME = 'mywlcg'

# Set up some useful paths for later
APP_PATH = os_path.abspath(os_path.split(__file__)[0])
PROJECT_PATH = os_path.abspath(os_path.join(APP_PATH, '..'))
CONFIG_FILE = PROJECT_PATH + '/config/mywlcgconf.ini'

# Load MyWLCG user defined settings
config = RawConfigParser()
read_files = config.read([CONFIG_FILE])
if not read_files:
    CONFIG_FILE = '/etc/mywlcg/mywlcg.ini'
    read_files = config.read([CONFIG_FILE])

def check_configured(section, option):
    """checks if a flag in the configuration is enabled"""
    conf = None
    try:
        conf = config.get(section, option).lower()
    except NoSectionError, e:
        pass
    except NoOptionError, e:
        pass
    except:
        raise ImproperlyConfigured(e)
    return conf == 'true'

ACE = check_configured('apps', 'ACE')
DAX = check_configured('apps', 'DATA_TRANSFER')
REPORTS = check_configured('apps', 'REPORTS')
TOPOLOGY = check_configured('apps', 'TOPOLOGY')
TRENDS = check_configured('apps', 'TRENDS')

try:
    DATABASES = {
        'default': {
                    'NAME': config.get('database', 'DATABASE_NAME'),
                    'ENGINE': 'django.db.backends.%s' % config.get('database', 'DATABASE_ENGINE'),
                    'USER': config.get('database', 'DATABASE_USER'),
                    'PASSWORD': config.get('database', 'DATABASE_PASSWORD'),
                    'HOST': config.get('database', 'DATABASE_HOST'),
                    'PORT': config.get('database', 'DATABASE_PORT')
        }
    }

    # General settings
    ADMIN_NAME = config.get('general', 'ADMIN_NAME')
    ADMIN_EMAIL = config.get('general', 'ADMIN_EMAIL')
    DEFAULT_PROFILE = config.get('general', 'DEFAULT_PROFILE')
    REGION = config.get('general', 'REGION')
    TIME_ZONE = config.get('general', 'TIME_ZONE')
    WLCG_TIER1_EMAILS = config.get('general', 'WLCG_TIER1_EMAILS')
    WLCG_TIER2_EMAILS = config.get('general', 'WLCG_TIER2_EMAILS')
    EGI_EMAILS = config.get('general', 'EGI_EMAILS')

    # Data Transfers
    if DAX:
        VO = float(config.get('general', 'DT_VO_OTHERS_LIMIT'))
        SRCSITE = float(config.get('general', 'DT_SRCSITE_OTHERS_LIMIT'))
        DSTSITE = float(config.get('general', 'DT_DSTSITE_OTHERS_LIMIT'))

    # Reports
    if REPORTS:
        AUTHENTIC_DNS = config.get('general', 'REPORT_AUTHENTIC_DNS')
        EGI_PROFILES = config.get('general', 'REPORT_EGI_PROFILES')
        WLCG_TIER1_HISTORY_PROFILES = config.get('general', 'REPORT_WLCG_TIER1_HISTORY_PROFILES')
        WLCG_TIER1_SUMMARY_PROFILES = config.get('general', 'REPORT_WLCG_TIER1_SUMMARY_PROFILES')
        WLCG_TIER1_VO_PROFILES = config.get('general', 'REPORT_WLCG_TIER1_VO_PROFILES')
        WLCG_TIER2_PROFILES = config.get('general', 'REPORT_WLCG_TIER2_PROFILES')
        WLCG_ALL_SITES_PROFILES = config.get('general', 'REPORT_VO_ALL_SITES_PROFILES')
        if config.has_option('general', 'REPORTS_HOST'):
            REPORTS_HOST = config.get('general', 'REPORTS_HOST')
        else:
            REPORTS_HOST = 'http://sam-reports.web.cern.ch/sam-reports/'

    # Database
    DB_LIMIT = int(config.get('general', 'DB_LIMIT'))
    THROTTLE = check_configured('general', 'THROTTLE')
    if THROTTLE:
        ACCESS_PERIOD = int(config.get('general', 'ACCESS_PERIOD'))
        NUMBER_OF_ACCESSES = int(config.get('general', 'NUMBER_OF_ACCESSES'))
    else:
        ACCESS_PERIOD = 1
        NUMBER_OF_ACCESSES = 1

    # Others
    SECRET_KEY = config.get('others', 'SECRET_KEY')
    DEBUG = check_configured('others', 'DEBUG')

except Exception, e:
    raise ImproperlyConfigured(e)

SAM_RELEASE_FILE = '/etc/sam-release'

TEMPLATE_DEBUG = DEBUG
ADMINS = (
    (ADMIN_NAME, ADMIN_EMAIL),
)
MANAGERS = ADMINS

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
if DEBUG:
    MEDIA_ROOT = os_path.join(APP_PATH, '/media')
else:
    MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin/media/'

# Make this unique, and don't share it with anybody.
# Using the one in the config file
#SECRET_KEY = ''

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

APPEND_SLASH = True

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    # 'django.contrib.sessions.middleware.SessionMiddleware',
    # 'django.middleware.csrf.CsrfViewMiddleware',
    # 'django.contrib.auth.middleware.AuthenticationMiddleware',
    # 'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'mywlcg.urls'

TEMPLATE_DIRS = (
    os_path.join(os_path.dirname(__file__), 'templates').replace('\\','/'),
)

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'mywlcg.base',
    'mywlcg.mdb',
    'mywlcg.services',
    'mywlcg.status',
    'mywlcg.gridmap',
    'mywlcg.ss',
    'mywlcg.metric',
    'mywlcg.filters',
    'mywlcg.tests',
    'mywlcg.ws',
    'mywlcg.sa',
    'mywlcg.dt',
    'mywlcg.reports',
    'mywlcg.trends',
    'mywlcg-atp-api',
    'mywlcg-atp-web'
)
INTERNAL_IPS = ('127.0.0.1',)

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': '/var/tmp/mywlcg_cache',
    }
}

if DEBUG:
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        }
    }
