from datetime import datetime
import StringIO
import csv as ocsv

from django.db.models.query_utils import Q
from django.shortcuts import render_to_response
from django.http import Http404
from django.http import HttpResponse
from django.utils import simplejson
from django.views.decorators.cache import cache_page

from mywlcg.mdb.models import Service
from mywlcg.mdb.models import MetricData
from mywlcg.mdb.models import Profile
from mywlcg.mdb.models import StatusChangeServiceProfile, StatusChangeMetricService
from mywlcg.filters.lib import parse_filters, parse_params
from mywlcg.filters.views import parse_selection, parse_common_filters
from mywlcg.base.views import path_prefix

def metric_name_length(num_columns):
    MAX = 16
    MIN = 2
    if num_columns >= MAX:
        return MIN
    else:
        return MAX - num_columns + MIN


@cache_page(600)
def index(request):
    # Profile
    # Results
    results = {}
    results = parse_selection(request, results)
    results = parse_common_filters(request, results)
    # Flavours
    pmm=Profile.objects.filter(id = results['profile'],isdeleted='N')
    if results.get('flavour'):
        flavour = map(lambda (idx, name): idx, results.get('flavour'))
        flavours = pmm[0].metricinstances.filter(atp_service_type_flavour__in=flavour)
    else:
        flavours = pmm[0].metricinstances
    flavours = flavours.order_by('atp_service_type_flavour__flavourname')\
	    		.distinct('atp_service_type_flavour').select_related(depth = 1)\
            		.values_list('atp_service_type_flavour', 'atp_service_type_flavour__flavourname')
    # Metrics to show
    flavour_columns = []
    for flavour in flavours:
        columns = pmm[0].metricinstances.filter(atp_service_type_flavour=flavour[0])\
            		.values('metric__name').order_by('metric__name')
        flavour_columns.append([flavour[0], flavour[1], len(columns)+2, columns])
    results['params'] = parse_params(request)
    results['flavour_columns'] = flavour_columns
    # Return
    return render_to_response('status_index.html', results)

@cache_page(600)
def selected(request, service_id, profile_id, unixtime):
    # Parameters
    vo=Profile.objects.get(id=profile_id,isdeleted='N').atp_vo.id
    flavour_id = Service.objects.get(id = service_id).flavour_id
    currentdatetime = datetime.utcfromtimestamp(int(unixtime)).strftime('%x %X UTC')
    filter=Q(vo=vo) | Q(vo__isnull=True)
    # Setup
    columns = Profile.objects.filter(id = profile_id,isdeleted='N')[0].metricinstances\
			.filter(atp_service_type_flavour=flavour_id)\
               		.distinct('metric').values('metric_id', 'metric__name')
    service = Service.objects.get(id = service_id)
    # Get results
    last_status = StatusChangeServiceProfile.objects.filter(\
        service = service_id, profile = profile_id,
        timestamp__lte = unixtime)\
        .order_by('-timestamp')\
        .values_list('timestamp', 'metricstatus__description')
    results = []
    service_status = ''
    if last_status:
        last_status = last_status[0]
        service_status = last_status[1]
        metrics = []
        for column in columns:
            res = MetricData.objects.filter(filter, service = service_id,\
                check_time__lte = unixtime, 
                metric = column['metric_id'])\
                .order_by('-check_time')\
                .values_list('metric_id', 'service_id', 'check_time', 'metricstatus__description')
            if len(res) > 0:
                metrics.append(res[0])
        temp2 = []
        for column in columns:
            found = False
            for metric in metrics:
                if column['metric_id'] == metric[0]:
                    temp2.append(metric)
                    found = True
                    break
            if not found:
                temp2.append([])
        temp = [last_status[1], service.hostname]
        temp.append(temp2)
        results.append(temp)
    
    data = {
        'columns': columns,
        'results': results,
        'currentdatetime': currentdatetime,
        'profile': profile_id,
        'vo_id': vo,
        'profilename': Profile.objects.get(id=profile_id,isdeleted='N').name,
        'service_status': service_status,
    }
    path_prefix(request,data) 
    return render_to_response('status_selected.html', data)


def get_results(request, type='flat'):
    # Initial setup
    try:
        flavour_id = int(request.GET.get("flavour_id"))
    except TypeError:
        raise Http404
    except ValueError:
        raise Http404
    profile,  queryset = parse_filters(request, Service.objects)
    queryset = queryset.filter(flavour=flavour_id).distinct()
    columns = Profile.objects.filter(id = profile,isdeleted='N')[0].metricinstances\
                        .filter(atp_service_type_flavour=flavour_id)\
                        .distinct('metric').values_list('metric_id', 'metric__name')
    #iTotalRecords = queryset.count()

    # Extracting results
    results = []
    temp = []
    for row in queryset:
        temp = [row.hostname]
        metric_in_profile = Profile.objects.filter(id = profile,isdeleted='N')[0]\
				.metricinstances.values_list('metric',flat=True)\
				.distinct('metric') 
        metrics =  row.metricdatalatest_set.filter(
                        metric__in = metric_in_profile)\
                        .values_list('metric_id', 'id', 'metricstatus__description')
        for col in columns:
            found = False
            for metric in metrics:
                if col[0] == metric[0]:
                    if type == 'xml':
                        temp.append([col[1], metric[2]])
                    else:
                        temp.append(metric[2])
                    found = True
                    break
            if not found:
                if type == 'xml':
                    temp.append([col[1], 'NOTFOUND'])
                else:
                    temp.append('NOTFOUND')
        results.append(temp)

    first = ['hostname']
    for col in columns:
        first.append(col[1])

    return [first, results]

@cache_page(600)
def json(request):
    first, results = get_results(request)
    results.insert(0, first)
    return HttpResponse(simplejson.dumps(results, indent=1),
        mimetype='application/javascript'
    )

@cache_page(600)
def xml(request):
    first, results = get_results(request, type='xml')
    return render_to_response(
        'status_xml',
        mimetype = 'application/xml',
        dictionary = {
            'results': results,
        }
    )

@cache_page(600)
def csv(request):
    first, results = get_results(request)
    out = StringIO.StringIO()
    wrtr = ocsv.writer(out, delimiter=',')
    wrtr.writerow(first)
    for result in results:
        wrtr.writerow(result)
    response = HttpResponse(out.getvalue(), mimetype='text/csv')
    response['Content-Disposition'] = 'attachment; filename=status.csv'
    return response

@cache_page(600)
def json_datatables(request):
    # Datatables GET parameters
    # http://www.datatables.net/usage/server-side
    iDisplayStart = int(request.GET.get("iDisplayStart") or 0)
    iDisplayLength = int(request.GET.get("iDisplayLength") or 10)
    iColumns = int(request.GET.get("iColumns") or -1)
    sSearch = request.GET.get("sSearch")
    bEscapeRegex = request.GET.get("bEscapeRegex")
    iSortingCols = int(request.GET.get("iSortingCols") or -1)
    sEcho = int(request.GET.get("sEcho") or -1)

    # Initial setup
    try:
        flavour_id = int(request.GET.get("flavour_id"))
    except TypeError:
        raise Http404
    except ValueError:
        raise Http404
    profile,  queryset = parse_filters(request, Service.objects)
    vo = Profile.objects.get(id=profile,isdeleted='N').atp_vo
    columns = Profile.objects.filter(id = profile,isdeleted='N')[0].metricinstances\
                        .filter(atp_service_type_flavour=flavour_id)\
                        .distinct('metric').values_list('metric_id',flat=True)\
			.order_by('metric__name')
    queryset = queryset.filter(flavour = flavour_id).distinct()
    iTotalRecords = queryset.count()

    # Search
    if (sSearch):
        queryset = queryset.filter(hostname__icontains = sSearch)

    # Extracting results
    iTotalDisplayRecords = queryset.count()
    if iDisplayLength and iDisplayLength != -1:
        queryset = queryset[iDisplayStart:iDisplayStart+iDisplayLength]
    results = []
    temp = []
    for row in queryset:
        temp = [row.id, row.hostname]
        filter=Q(vo=vo) | Q(vo__isnull=True)
        metric_in_profile = Profile.objects.filter(id = profile,isdeleted='N')[0]\
                                .metricinstances.values_list('metric',flat=True)\
                                .distinct('metric')
        metrics =  row.metricdatalatest_set.filter(filter,
                        metric__in = metric_in_profile)\
                        .distinct()\
                        .values_list('metric_id', 'id', 'metricstatus__description')
                        #.values_list('metric_id', 'id', 'metricstatus__description', 'fqan')
        temp2 = []
        for col in columns:
            found = False
            for metric in metrics:
                #if col == metric[0] and (str(metric[3]) == fqan or metric[3] == None):
                if col == metric[0]:
                    temp2.append(metric)
                    found = True
                    break
            if not found:
                temp2.append([])
        temp.append(temp2)
        results.append(temp)

    # Render
    data = {
        'iTotalRecords': iTotalRecords,
        'iTotalDisplayRecords': iTotalDisplayRecords,
        'sEcho': sEcho,
        'results': results
    }
    path_prefix(request,data) 
    return render_to_response('status_json', data)
