from django.test.client import Client

from mywlcg import settings
from mywlcg.mdb import models

test_list = [
    'test_model_introspection',
    'test_urls',
    'test_atp',
    'test_mrs',
]

test_web = [
     'test_ws',
]

tables_list = [
    'ACE_H_SITE_AVAILABILITY',
    'ACE_D_SITE_AVAILABILITY',
    'ACE_W_SITE_AVAILABILITY',
    'ACE_M_SITE_AVAILABILITY',
    'ACE_H_FLAVOUR_AVAILABILITY',
    'ACE_D_FLAVOUR_AVAILABILITY',
    'ACE_W_FLAVOUR_AVAILABILITY',
    'ACE_M_FLAVOUR_AVAILABILITY',
    'ACE_H_AVAILABILITY',
    'ACE_D_AVAILABILITY',
    'ACE_W_AVAILABILITY',
    'ACE_M_AVAILABILITY',
    'AceServiceStatus',
    'AceSiteStatus',
    'AceAlgorithmProfileMap',
    'AceAlgorithm',
    'AceFlavourStatus',
    'Country',#To check ATP
    'Downtime',
    'Fqan',
    'FtsHourlySummary',
    'FtsDailySummary',
    'FtsWeeklySummary',
    'FtsMonthlySummary',
    'Groups',
    'GroupType',
    'Infrastructure',#To check ATP
    'Metric',
    'MetricData',
    'MetricDataLatest',
    'MetricdataSpool',
    'MetricstoreCurrLoadM',
    'MetricstoreCurrLoadH',
    'MetricstoreCurrLoadHDET',
    'MetricDetails',#To check MRS
    'MetricStatus',
    'Profile',
    'Service',
    'ServiceGroup',
    'Service_Site',
    'ServiceTypeFlavour',
    'Site',
    'SiteGroup',
    'SiteStoreCap',
    'StatusChangeServiceProfile',
    'StatusChangeServiceProfileLatest',
    'StatusChangeMetricService',
    'Vo',
    'VoGroup',
    'VoServiceGroup',
]

def test_item(name, returncode, output):
    return { 'name': name, 'returncode': returncode, 'output': output }


def test_model_introspection(request):
    failed_models = []
    empty_models = []
    tested_models = []
    returncode = 0
    for model_name in tables_list:
       if (settings.ACE or not model_name.lower().startswith('ace'))\
		and (settings.DAX or not model_name.lower().startswith('fts')):
            tested_models.append(model_name)
            model = getattr(models, model_name)
            obj = []
            try:
                obj = list(model.objects.all()[0:1])
            except Exception:
                returncode += 1
                failed_models.append(model_name)
            if len(obj) == 0:
                empty_models.append(model_name)

    output = ''
    if len(failed_models):
        output += "<p>%s models failed: %s</p>" % (len(failed_models), failed_models,)
    else:
        output += "<p>No models failed</p>"
    output += "<p>%s empty models found: %s</p>" % (len(empty_models), empty_models,)
    output += "<p>%s models tested: %s</p>" % (len(tested_models), tested_models,)

    return test_item('DB Model Introspection', returncode, output)


def test_urls(request):
    test_urls = [
        {
            'returncodes': [200,], # OK
            'urls': [
                '/',
                '/blog/',
                '/ss/',
                '/ss/json/',
                '/metric/',
                '/metric/json/',
                '/gridmap/',
                '/gridmap/json/',
            ]
        },
        {
            'returncodes': [200, 301, 302,], # OK, REDIRECT
            'urls': [
            ]
        },
        {
            'returncodes': [404,], # Not found
            'urls': [
                '/ss/graph/',
                '/ss/a/a/',
                '/ss/a/a/21-10-2010/',
                '/ss/a/a/21-10-2010/22-10-2010/',
            ]
        }
    ]
    if settings.ACE:
       test_urls[0]['urls']=test_urls[0]['urls']+['/sa/',]
    if settings.DAX:
       test_urls[0]['urls']=test_urls[0]['urls']+['/dt/vo/','/dt/sp/','/dt/dp/']
    if settings.REPORTS:
       test_urls[0]['urls']=test_urls[0]['urls']+['/reports/']
    # do not test this view on MyEGI deployments
    # This is a dirty hack that should be removed once the settings for
    # mywlcg and myegi are splitted
    if 'myegi' not in request.path and settings.TRENDS:
       test_urls[0]['urls']=test_urls[0]['urls']+['/trends/']
    returncode = 0
    output = 'Failed URLs:<ul>'
    c = Client()
    for test_url in test_urls:
        for url in test_url['urls']:
            try:
                response = c.get(url)
                if not response.status_code in test_url['returncodes']:
                    returncode += 1
                    output += "<li>%s</li>" % (url,)
            except:
                returncode += 1
                output += "<li>%s</li>" % (url,)
    output += '</ul>'
    return test_item('Common URLs', returncode, output)


def check_empty_models(non_empty_models):
    empty_models = []

    for model_name in non_empty_models:
        if not model_name.startswith('__') and not model_name in ['YESNO_CHOICE', 'models', 'Model']:
            model = getattr(models, model_name)
            try:
                model.objects.all()[0]
            except IndexError:
                empty_models.append(model_name)
            except Exception:
                pass

    return empty_models


def test_atp(request):
    output = ''
    return_code = 0

    empty_tables_after_installation = check_empty_models([
        'Infrastructure',
        'Country',
        'ServiceTypeFlavour',
        # Not in the model: 'Synchronizer',
        'Groups',
        'GroupType',
    ])

    empty_tables_after_first_synchronization = check_empty_models([
        'Vo',
        'Service',
        'Site',
        'Downtime',
        # No way to check empty relations from models: 'Service_Group',
        # No way to check empty relations from models: 'Service_Vo',
        # Not in the model: 'SynchronizerLastrun'
        # No way to check empty relations from models: 'Site_Downtime',
        'SiteGroup',
    ])

    if len(empty_tables_after_installation) > 0:
        return_code += 1
        output += "ERROR: MyWLCG found %s empty tables that should contain data\
            after ATP installation.<br />" % (len(empty_tables_after_installation),)
        output += "Empty tables: %s<br />" % (empty_tables_after_installation,)

    if len(empty_tables_after_first_synchronization) > 0:
        return_code += 2
        output += "ERROR: MyWLCG found %s empty tables that should contain data\
            after first ATP synchronization.<br />" % (len(empty_tables_after_first_synchronization),)
        output += "Empty tables: %s<br />" % (empty_tables_after_first_synchronization,)

    return test_item('Agregated Topology Provider (ATP)', return_code, output)


def test_mddb(request):
    output = ''
    return_code = 0

    empty_tables_after_first_synchronization = check_empty_models([
        'Metric',
        'Profile',
        'ProfileMetricMap',
        'FlavourMetricMap',
    ])

    if len(empty_tables_after_first_synchronization) > 0:
        return_code += 2
        output += "ERROR: MyWLCG found %s empty tables that should contain data\
            after first MDDB synchronization.<br />" % (len(empty_tables_after_first_synchronization),)
        output += "Empty tables: %s<br />" % (empty_tables_after_first_synchronization,)

    return test_item('Metric Description Database (MDDB)', return_code, output)


def test_mrs(request):
    output = ''
    return_code = 0

    empty_tables_after_installation = check_empty_models([
        'MetricStatus',
    ])

    # Models do not exist
    #empty_tables_after_initial_load = check_empty_models([
    #    'SUPPORTED_PROFILES',
    #    'FQANS_IN_SUPPORTED_SERVICES',
    #    'METRICS_IN_SUPPORTED_SERVICES'
    #])

    # MRS post sync checks should not be done in MyWLCG
    # empty_tables_after_first_synchronization = check_empty_models([
    #     # Not in the model: 'FQAN',
    #     # Not in the model: 'GATHEREDAT',
    #     'MetricData',
    #     'MetricDataLatest',
    #     'MetricDetails',
    #     'StatusChangeMetricService',
    #     # Not in the model: 'STATUSCHANGE_METRIC_SERV_FQAN',
    #     'StatusChangeServiceProfile',
    #     # Not in the model: 'STATUSCHANGE_SERV_PROFILE_FQAN',
    #     # Not in the model: 'TESTED_SERVICES',
    #     # Not in the model: 'TESTED_SERVICES_FQAN',
    # ])

    #empty_tables_after_one_week = check_empty_models([
        # Not in the model: 'MON_FREQUENCY_STATISTICS',
    #])

    if len(empty_tables_after_installation) > 0:
        return_code += 1
        output += "ERROR: MyWLCG found %s empty tables that should contain data\
            after MRS installation.<br />" % (len(empty_tables_after_installation),)
        output += "Empty tables: %s<br />" % (empty_tables_after_installation,)

    # if len(empty_tables_after_first_synchronization) > 0:
    #     return_code += 2
    #     output += "ERROR: MyWLCG found %s empty tables that should contain data\
    #         after first MRS synchronization.<br />" % (len(empty_tables_after_first_synchronization),)
    #     output += "Empty tables: %s<br />" % (empty_tables_after_first_synchronization,)

    return test_item('Metric Result Store (MRS)', return_code, output)


def test_ws(request):
    sitename = models.Site.objects.values_list('sitename', flat=True)[0]
    service = models.Service.objects.values('hostname')[0]

    test_urls = [
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/site_names/',
                '/sam-pi/site_names/?output=xml',
                '/sam-pi/site_names/?output=json',
                '/sam-pi/site_names/?infrast_name=OSG',
                '/sam-pi/site_names/?infrast_type=Production',
                '/sam-pi/site_names/?is_certified=Y',
                '/sam-pi/site_names/?is_certified=N',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/service_types_and_flavours/',
                '/sam-pi/service_types_and_flavours/?output=xml',
                '/sam-pi/service_types_and_flavours/?output=json',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/status/',
                '/sam-pi/status/?output=json',
                '/sam-pi/status/?output=xml',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/service_maintenance/?host_name=%s&output=xml' % (service['hostname'],),
                '/sam-pi/service_maintenance/?host_name=%s&output=json' % (service['hostname'],),
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/service_types_and_flavours_per_service/',
                '/sam-pi/service_types_and_flavours_per_service/?output=xml',
                '/sam-pi/service_types_and_flavours_per_service/?output=json',
                '/sam-pi/service_types_and_flavours_per_service/?host_name=%s' % (service['hostname'],),
                '/sam-pi/service_types_and_flavours_per_service/?site_name=%s' % (sitename,),
                '/sam-pi/service_types_and_flavours_per_service/?infrast_name=OSG',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/vo_names/',
                '/sam-pi/vo_names/?output=xml',
                '/sam-pi/vo_names/?output=json',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/vos_per_service/',
                '/sam-pi/vos_per_service/?output=xml',
                '/sam-pi/vos_per_service/?output=json',
                '/sam-pi/vos_per_service/?site_name=%s' % (sitename,),
                '/sam-pi/vos_per_service/?vo_name=ops',
                '/sam-pi/vos_per_service/?infrast_name=OSG',
                '/sam-pi/vos_per_service/?service_type=CE',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/groups_defined_by_vos/',
                '/sam-pi/groups_defined_by_vos/?output=xml',
                '/sam-pi/groups_defined_by_vos/?output=json',
                '/sam-pi/groups_defined_by_vos/?vo_name=ops',
                '/sam-pi/groups_defined_by_vos/?group_type=LHCb_Site',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/services_defined_in_groups/?vo_name=ops&output=xml',
                '/sam-pi/services_defined_in_groups/?vo_name=lhcb&output=json',
                '/sam-pi/services_defined_in_groups/?vo_name=lhcb&group_name=LCG.BHAM-HEP.uk',
                '/sam-pi/services_defined_in_groups/?vo_name=lhcb&group_type=LHCb_Site',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/service_types_and_flavours_defined_in_groups/?vo_name=ops',
                '/sam-pi/service_types_and_flavours_defined_in_groups/?vo_name=ops&output=xml',
                '/sam-pi/service_types_and_flavours_defined_in_groups/?vo_name=ops&output=json',
                '/sam-pi/service_types_and_flavours_defined_in_groups/?vo_name=ops&group_name=LCG.BHAM-HEP.uk',
                '/sam-pi/service_types_and_flavours_defined_in_groups/?vo_name=ops&group_type=LHCb_Site',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/metrics_in_profiles/?vo_name=ops',
                '/sam-pi/metrics_in_profiles/?vo_name=ops&output=xml',
                '/sam-pi/metrics_in_profiles/?vo_name=ops&output=json',
                '/sam-pi/metrics_in_profiles/?vo_name=ops&profile_name=ALICE_CRITICAL',
                '/sam-pi/metrics_in_profiles/?vo_name=ops&service_type_flavour=CREAM-CE',
                '/sam-pi/metrics_in_profiles/?vo_name=ops&service_type=CE',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/metrics_history_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&start_time=2010-10-01T13:22:15Z&end_time=2010-10-02T00:00:00Z',
                '/sam-pi/metrics_history_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&start_time=2010-10-01T13:22:15Z&end_time=2010-10-02T00:00:00Z&output=json',
                '/sam-pi/metrics_history_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&start_time=2010-10-01T13:22:15Z&end_time=2010-10-02T00:00:00Z&output=xml',
                '/sam-pi/metrics_history_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&start_time=2010-10-01T13:22:15Z&end_time=2010-10-02T00:00:00Z&service_flavour=CREAM-CE',
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
            ]
        },
        {
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/latest_metric_results_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL',
                '/sam-pi/latest_metric_results_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&output=xml',
                '/sam-pi/latest_metric_results_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&output=json',
                '/sam-pi/latest_metric_results_in_profile/?vo_name=lhcb&profile_name=ALICE_CRITICAL&service_flavour=CREAM-CE',
            ]
        },
        {
            'ACE': True,
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/status_of_service_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice',
                '/sam-pi/status_of_service_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&output=xml',
                '/sam-pi/status_of_service_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&output=json',
                '/sam-pi/status_of_service_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&service_type=Site-BDII',
                '/sam-pi/status_of_service_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&service_type_flavour=Site-BDII',
            ]
        },
        {
            'ACE': True,
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/status_of_site_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&group_type=Site',
                '/sam-pi/status_of_site_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&group_type=Site&output=xml',
                '/sam-pi/status_of_site_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&group_type=Site&output=json',
                '/sam-pi/status_of_site_in_profile/?profile_name=ALICE_CRITICAL&vo_name=alice&group_type=Site&group_name=INFN-FERRARA',
            ]
        },
        {
            'ACE': False,
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/downtimes/?start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z',
            ]
        },
        {
            'ACE': True,
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/service_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=MONTHLY',
                '/sam-pi/service_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=DAILY',
                '/sam-pi/service_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=HOURLY',
                '/sam-pi/service_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=WEEKLY',
            ]
        },
        {
            'ACE': True,
            'returncodes': [200,], # OK
            'urls': [
                '/sam-pi/group_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=MONTHLY&group_type=Site',
                '/sam-pi/group_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=DAILY&group_type=Site',
                '/sam-pi/group_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=HOURLY&group_type=Site',
                '/sam-pi/group_availability_in_profile/?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=WEEKLY&group_type=Site',
            ]
        },
        {
            'returncodes': [500,], # OK, REDIRECT
            'urls': [
            #    '/sam-pi/service_maintenance',
            #    '/sam-pi/services_defined_in_groups',
            #    '/sam-pi/service_types_and_flavours_defined_in_groups',
            #    '/sam-pi/metrics_in_profiles',
            #    '/sam-pi/metrics_history_in_profile',
            #    '/sam-pi/metrics_history_in_profile?profile_name=ALICE_CRITICAL',
            #    '/sam-pi/metrics_history_in_profile?vo_name=alice',
            #    '/sam-pi/metrics_history_in_profile?profile_name=ALICE_CRITICAL&vo_name=alice',
            #    '/sam-pi/metrics_history_in_profile?start_time=0',
            #    '/sam-pi/metrics_history_in_profile?end_time=0',
            #    '/sam-pi/metrics_history_in_profile?profile_name=ALICE_CRITICAL&vo_name=alice&start_time=0',
            #    '/sam-pi/metrics_history_in_profile?profile_name=ALICE_CRITICAL&vo_name=alice&end_time=0',
            #    '/sam-pi/metric_output_details',
            #    '/sam-pi/latest_metric_results_in_profile',
            #    '/sam-pi/latest_metric_results_in_profile?profile_name=ALICE_CRITICAL',
            #    '/sam-pi/latest_metric_results_in_profile?vo_name=alice',
            #    '/sam-pi/status_of_service_in_profile',
            #    '/sam-pi/status_of_site_in_profile',
            #    '/sam-pi/service_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=X',
            #    '/sam-pi/service_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z',
            #    '/sam-pi/service_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL',
            #    '/sam-pi/group_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z&type=X',
            #    '/sam-pi/group_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL&start_time=2011-01-30T13:22:15Z&end_time=2011-02-01T20:22:15Z',
            #    '/sam-pi/group_availability_in_profile?vo_name=cms&profile_name=CMS_CRITICAL',
            ]
        }
    ]
    returncode = 0
    tested_urls_count = 0
    output = 'Failed URLs:<ul>'
    c = Client()
    for test_url in test_urls:
        if settings.ACE or not 'ACE' in test_url:
            for url in test_url['urls']:
                tested_urls_count += 1
                try:
                    response = c.get(url)
                    if not response.status_code in test_url['returncodes']:
                        returncode += 1
                        output += "<li><a href=\"%s\">%s</a></li>" % (url, url)
                except:
                    returncode += 1
                    output += "<li><a href=\"%s\">%s</a></li>" % (url, url)
    output = '<p>Tested URLs: ' + str(tested_urls_count) + '</p>' + output + '</ul>'
    return test_item('Web Services', returncode, output)
