from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.views.decorators.cache import cache_page
from mywlcg.base.views import path_prefix
from mywlcg.tests import tests

#@cache_page(3600)
def index(request):
    results={}
    results['test'] = request.GET.get("test") or 0
    path_prefix(request,results)
    return render_to_response('tests_index.html',results)

def results(request, type='html'):
    # Execute tests and make statistic calculations
    test = request.GET.get("test")
    test_results = []
    tests_ok = 0
    tests_failed = 0
    if test=='web':
       tests_todo=tests.test_web
    else:
       tests_todo=tests.test_list
    for test in tests_todo:
        res = getattr(tests, test)(request)
        if res['returncode'] == 0:
            tests_ok += 1
        else:
            tests_failed += 1
        test_results.append(res)
    
    # Output
    if type == 'html':
        results = {
            'results': test_results,
            'tests_total': tests_ok + tests_failed,
            'tests_ok': tests_ok,
            'tests_failed': tests_failed,
        }
        path_prefix(request,results)
        response = render_to_response('tests_results.html', results)
    elif type == 'text':
        response = HttpResponse(_generate_text_results(test_results), mimetype="text/plain")
        response['Content-Disposition'] = 'filename=mywlcg-log.txt'
    elif type == 'nagios':
        return_code = 0
        return_msg = ['OK', 'WARNING']
        for test_result in test_results:
            if str(test_result['returncode']) != str(0):
                return_code = 1
                break
        msg = "Check %s" % (request.build_absolute_uri('../../'),)
        output = "%s %s - %s" % ('MyWLCGCheck', return_msg[return_code], msg)
        response = HttpResponse(output, mimetype="text/plain")
    response['Cache-Control'] = 'no-cache'
    return response


def _generate_text_results(tests):
    results = ''
    i = 1
    tests_ok = 0
    tests_failed = 0
    for test in tests:
        # Text
        res = 'Test ' + str(i) + ': ' + test['name'] + "\n"
        res += 'Return code: ' + str(test['returncode'])
        res += "\n" + test['output'] + "\n\n"
        results += res
        # Calculations
        if test['returncode'] == 0:
            tests_ok += 1
        else:
            tests_failed += 1
        i += 1
    res = "------------------------------------------------------------------H\n"
    res += "Total tests: " + str(tests_ok + tests_failed) + "\n"
    res += "OK: " + str(tests_ok) + "\n"
    res += "Failed: " + str(tests_failed) + "\n"
    results += res
    return results
