
import calendar
import time
from datetime import timedelta, datetime
from dateutil.relativedelta import relativedelta

import urllib
import simplejson
from xml.dom import minidom
from xml.parsers.expat import ExpatError

from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.core.urlresolvers import reverse
from django.template import loader, Context
from django.shortcuts import render_to_response
from django.db.models import Sum
from django.db.models.query_utils import Q
from django.core.exceptions import ObjectDoesNotExist

from mywlcg.filters.lib import get_grouptype 
from mywlcg.filters.views import parse_filters_trends
from mywlcg.mdb.models import (SnapshotTopologyView, ACE_M_SITE_AVAILABILITY,
                               Profile, Groups, Vo, VoServiceGroup)


vos = ['ALICE','ATLAS','CMS','LHCb']
vos_lower = ['alice','atlas','cms','lhcb']
EGI_ACCOUNTING_API_URL = 'http://accounting.egi.eu/custom_xml.php'

PLOT_AVAILABILITY = 1
PLOT_RELIABILITY = 2

PLOT_NUMBER_OF_JOBS = 1
PLOT_CPU_TIME_USED = 3
PLOT_CPU_WALL_TIME_USED = 5
PLOT_CPU_EFFICENCY = 7
PLOT_PERC_HOURS_USED = 9


def setdate(request, results):

    # Fixed dates
    if results.get('dateorperiod') == 'pd':
        fixed_period = results.get('period')
        if fixed_period == 'p3':
            delta = relativedelta(months=3)
        elif fixed_period == 'p12':
            delta = relativedelta(months=12)
        elif fixed_period == 'p24':
            delta = relativedelta(months=24)
        else:
            delta = relativedelta(months=6)

        enddate = datetime.utcnow()
        startdate = (enddate - delta).replace(day=1)  # use start of the month

    # Custom dates
    else:
        enddate = results.get('enddate')
        startdate = results.get('startdate')

        try:
            enddate = datetime(*time.strptime(enddate, '%m-%Y')[0:3])
            startdate = datetime(*time.strptime(startdate, '%m-%Y')[0:3])
        except (ValueError, TypeError):
            enddate = datetime.utcnow()
            startdate = enddate - relativedelta(months=1)

    results['starttime'] = startdate
    results['endtime'] = enddate
    return results


def index(request):
    return HttpResponseRedirect(reverse('mywlcg.trends.views.tab', kwargs={'tab': 'eu'}))


def tab(request, tab):
    # hide this view from MyEGI deployment
    # This is a dirty hack that should be removed once the settings for
    # mywlcg and myegi are splitted
    if 'myegi' in request.path:
        raise Http404

    if tab not in ('at', 'eu'):
        return render_to_response('../../base/templates/400.html', {'error': 'Page Does Not Exist'})

    results = parse_filters_trends(request, {'tab': tab})
    try:
        results = setdate(request, results)
    except ValueError, ve:
        return render_to_response('../../base/templates/400.html', {'error': ve})

    if not results['startdate'] or not results['enddate']:
        now = datetime.utcnow().replace(day=1)
        results['enddate'] = now.strftime('%m-%Y')
        results['startdate'] = (now - relativedelta(months=1)).strftime('%m-%Y')

    if tab == 'eu':
        results['vos'] = Vo.objects.filter(voname__in=vos_lower).values_list('id','voname')
        results['vo'] = int(request.GET.get("vo") or Vo.objects.get(voname='cms').id)
        results['sites'] = ['all']
        results['countries'] = ['all']
        year = results['starttime'].year

        def resources(column, field, year, tier):
            query_url="http://wlcg-rebus.cern.ch/apps/pledges/resources/%d/%d/json" % (year, tier)
            result = simplejson.load(urllib.urlopen(query_url))
            for row in result:
                yield row[column]

        # populate Tier-0 sites
        for site in resources('Federation', 'sites', year, 0):
            if site not in results['sites']:
                results['sites'].append(site)
        # populate Tier-1 sites
        for site in resources('Federation', 'sites', year, 1):
            if site not in results['sites']:
                results['sites'].append(site)
        # populate Countries
        for country in resources('Country', 'countries', year, 2):
            if country not in results['countries']:
                results['countries'].append(country)

        # set selected site and country
        results['site'] = request.GET.get("site") or results['sites'][0]
        results['country'] = request.GET.get("country") or results['countries'][0]

    if results['view'] == PLOT_PERC_HOURS_USED:
        results['stacked'] = 0
    else:
        results['stacked'] = 1
    
    return render_to_response('trends_index.html', results)


def gettitle(view):
    if view == PLOT_NUMBER_OF_JOBS:
        query = 'njobs'
        query1 = 'sum_njobs'
        view_type = "Number of Jobs"
        unit = 'K Jobs'
        factor = 0.001
    elif view == PLOT_CPU_TIME_USED:
        query = 'normcpu'
        query1 = 'norm_cpu'
        view_type = "CPU Time used"
        factor = 0.004
        unit = 'K Hours'
    elif view == PLOT_CPU_WALL_TIME_USED:
        query = 'normelap'
        query1 = 'norm_elapsed'
        view_type = "CPU Wall Clock Time used"
        factor = 0.004
        unit = 'K Hours'
    elif view == PLOT_CPU_EFFICENCY:
        query = 'cpueff'
        query1 = ('sum_cpu', 'sum_wct')
        factor = 100
        view_type = "CPU Efficiency"
        unit = '%'
    else:
        query = 'normelap'
        query1 = 'norm_elapsed'
        factor = 4.0
        view_type = "Pledged Hours Used"
        unit = '%'
    return (view_type, query, query1, factor, unit)


def heading(request, tab):
    # Availability Trends
    if tab == 'at':
        group = request.GET.get("group") or "tier0"
        view = request.GET.get("view") or '1'
        labels = {
            '1': 'Availability',  # plot view labels
            '2': 'Reliability',
            'tier0': 'Tier-0',  # site group labels
            'tier1': 'Tier-1 Sites',
            'tier01': 'Tier-0 and Tier-1 Sites',
            'tier2': 'Tier-2 Sites'
        }
        title = '%s %s' % (labels.get(group), labels.get(view))

    # Experiment Usage
    else:
        view = int(request.GET.get("view") or 1)
        group = request.GET.get("group", '')
        site = request.GET.get("site")
        country = request.GET.get("country") 
        vo = request.GET.get('vo')
        title, q, q1, f, u = gettitle(view)

        labels = {
            'tier0': 'CH-CERN',
            'tier1': 'All Tier-1 Sites',
            'tier01': 'All Tier-0 and Tier-1 Sites',
            'tier2': 'All Countries (Tier-2 Sites)'
        }
        if group in ('tier1', 'tier01') and site != 'all':
            plotted_region = site
        elif group == 'tier2' and country != 'all':
            plotted_region = country
        else:
            plotted_region = labels[group]

        if view != PLOT_PERC_HOURS_USED:
            try:
                voname = Vo.objects.get(id=vo).voname
            except ObjectDoesNotExist:
                voname = 'CMS'
            title += ' (%s)' % voname.upper()
        title += ': %s' % plotted_region
    return title
    return render_to_response('trends_heading', {"title": title})


def get_node_text(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc)


def calculatePledges(site, stime, etime, group):
    if group == 'tier0':
        tiers = ['0']
        column = 'Federation'
        if site == 'CERN-PROD':
            site = 'CH-CERN'
    elif group == 'tier1':
        tiers = ['1']
        column = 'Federation'
    elif group == 'tier01':
        tiers = ['0', '1']
        column = 'Federation'
    elif group == 'tier2':
        tiers = ['2']
        column = "Country"

    pledge = {}
    while stime.year - 1 <= etime.year:
        pledge[stime.year-1]={}
        for tier in tiers:
            query_url = "http://wlcg-rebus.cern.ch/apps/pledges/resources/%s/%s/json" % (str(stime.year-1), tier)
            result = simplejson.load(urllib.urlopen(query_url))
            for row in result:
                if row.get('PledgeType') == 'CPU':
                    for vo in vos:
                        if site in (row[column], 'all'):
                            new_total = (row[vo] * 720) or 0 
                            try:
                                last_total = pledge[stime.year - 1][vo.lower()]
                            except KeyError:
                                last_total = 0
                            pledge[stime.year - 1][vo.lower()] = last_total + new_total
        stime = stime + relativedelta(years=1)
    return pledge


def get_tier_sites(vo, profile_id, max_topology_ts, tier_group):
    max_ts = SnapshotTopologyView.objects.filter(
                 snapshotts__lt=max_topology_ts
             ).values_list(
                 'snapshotts', flat=True
             ).order_by('-snapshotts')
    try:
        max_ts = max_ts[0]
    except IndexError:
        return []

    try:
        profile = Profile.objects.get(id=profile_id)
    except ObjectDoesNotExist:
        return []

    tiers = {'tier0': ['Tier-0'],
             'tier1': ['Tier-1'],
             'tier01': ['Tier-0', 'Tier-1'],
             'tier2': ['Tier-2']}
    return SnapshotTopologyView.objects.filter(
        snapshotts=max_ts,
        tier__in=tiers[tier_group],
        profilename=profile.name
    )


def json(request, tab):
    # filters
    view = int(request.GET.get("view") or 1)
    group = request.GET.get("group") or 'tier0'
    vo = int(request.GET.get("vo") or 0)
    site = request.GET.get("site")
    country = request.GET.get("country")
    profile = int(request.GET.get("profile") or 0)

    csv = int(request.GET.get("csv") or 0)
    best = [int(request.GET.get("bestn") or 20), int(request.GET.get("bestm") or 50), 100]

    # dates
    starttime = request.GET.get("starttime")
    endtime = request.GET.get("endtime")
    try:
        stime = datetime(*time.strptime(starttime, '%m-%Y')[0:3])
        etime = datetime(*time.strptime(endtime, '%m-%Y')[0:3])
    except (ValueError, TypeError):
        etime = datetime.utcnow()
        stime = etime - relativedelta(months=1)

    # plot settings
    unit = '%'
    dateformat = "%m-%Y"
    results = {
        'mapdata': {
            'unit': None,  # set later on
            'rows': [],
            'cols': [{
                'label': 'Date (month-year)',
                'type': 'string'
            }],
        },
        'title': heading(request, tab),
        'ytitle': None,  # set later on
        'xtitle': 'Date',
        'slanted_text': 0,
        'slanted_angle': 0
    }

    try:
        voname = Vo.objects.get(id=vo).voname.lower()
    except ObjectDoesNotExist:
        voname = 'cms'

    # Availability Trends
    if tab == "at":
        results['ytitle'] = "Average (%)"
        results['mapdata']['unit'] = '%'

        first_month = stime.replace(day=1)
        last_month = etime.replace(day=1)
        group_type = get_grouptype(Vo.objects.get(id=vo).voname)

        tier_sites = get_tier_sites(vo, profile, last_month, group)
        nplots = (2, )
        if group == 'tier01':
            nplots = (0, 1, 2, )

        availabilities = ACE_M_SITE_AVAILABILITY.objects.filter(
            # values in the given months
            TIMESTAMP__range=(first_month, last_month),
            # for the selected profile
            PROFILE=profile,
            # contained in the correct tier group
            GROUPING__in=tier_sites.values('loggroupingid'),
            # avoiding non applicable availabilities
            AVAILABILITY__gte=0
        )
        nsites = availabilities.values('GROUPING__id').distinct().count()
        results['gp'] = str(availabilities.count()) or ''

        if view == PLOT_AVAILABILITY:
            avl_ranking = availabilities.annotate(avl=Sum('AVAILABILITY'))
        else:
            avl_ranking = availabilities.annotate(avl=Sum('RELIABILITY'))
        avl_ranking = avl_ranking.order_by('-avl').values('GROUPING', 'avl')

        months = []
        while first_month <= last_month:
            months.append(first_month)
            first_month += relativedelta(months=1)

        def get_best_n_sites(nbest):
            best_nsites = int(round(nsites * (float(nbest) / 100.0), 0))
            return max(best_nsites, 1)
        best_nsites = map(get_best_n_sites, best)

        for month in months:
            results['mapdata']['rows'].append([])
            tmstr = str(month.strftime(dateformat))
            results['mapdata']['rows'][-1].append(tmstr)

            for nbest in best_nsites:
                site_avls = availabilities.filter(
                    TIMESTAMP=month,
                    GROUPING__in=[g['GROUPING'] for g in avl_ranking[0:nbest]],
                    AVAILABILITY__gte=0
                )

                total = 0.0
                for avl in site_avls:
                    total += avl.AVAILABILITY * 100
                average = round(total / max(len(site_avls), 1), 2)
                results['mapdata']['rows'][-1].append(average)

        for i in nplots:
            results['mapdata']['cols'].append({
                'label': 'Average %d%% best sites' % best[i],
                'type': 'number'
            })

    # Experiments Usage
    else:
        ndatas = []  # main values provided for each region
        regions = {}  # region name / ndatas idx mapping
        resulted_regions = []  # regions that have provided a value

        j = 0
        k = 0
        results['gp'] = ''
        plot_y_title, query, query1, factor, unit = gettitle(view)
        results['ytitle'] = "%s (%s)" % (plot_y_title, unit)
        results['mapdata']['unit'] = unit

        # api option method depending on group
        api_options = {
            'tier0': 'CERN-PROD',
            'tier1': site,
            'tier01': site,
            'tier2': country,
        }

        # api parameters depending on group
        api_parameters = {
            # (option, function_of, specific sitename or country, api source tag)
            'tier0': ('TIER1', 'TIER1', 'CH-CERN', 'region'),
            'tier1': ('TIER1', 'TIER1', api_options[group], 'region'),
            'tier01': ('TIER1', 'TIER1', api_options[group], 'region'),
            'tier2': ('COUNTRY_T2', 'COUNTRY_T2', api_options[group], 'region'),
        }

        option, function_of, sitename, source = api_parameters[group]

        if view == PLOT_PERC_HOURS_USED:
            pdata = calculatePledges(sitename, stime, etime, group)

        icount = 0
        while stime <= etime:
            tmstr = str(stime.strftime(dateformat))
            results['mapdata']['rows'].append([])
            results['mapdata']['rows'][j].append(tmstr)

            params = {
                'sYear': stime.year,
                'sMonth': stime.month,
                'eYear': stime.year,
                'eMonth': stime.month,
                'query': query,
                'option': option,
                'yRange': function_of,
                'xRange': 'VO',
                'nodteam': 0,
                'groupVO': 'custom',
                'listVO': ';'.join(vos_lower + ['local-%s' % v for v in vos_lower]),
                'groupData': 'Production',
                'newCateg': '',
            }
            url = '%s?%s' % (EGI_ACCOUNTING_API_URL, urllib.urlencode(params))
            try:
                dom = minidom.parse(urllib.urlopen(url))
                data_list = dom.getElementsByTagName('data')
            except (ExpatError, IOError):
                data_list = []

            if view == PLOT_PERC_HOURS_USED:
                for v in vos:
                    results['mapdata']['rows'][j].append(0)
                if stime.month < 4:
                    year = stime.year - 1
                else:
                    year = stime.year

            if len(data_list):
                results['gp'] = '1'

            # reset total values for average calculations
            if view == PLOT_PERC_HOURS_USED:
                pledge_added_vos = set()
                tdata = 0
                tcdata = 0
                tpdata = 0

            for data in data_list:
                data_vo = get_node_text(data.getElementsByTagName('LCGUserVO')[0].childNodes).lower()
                region = get_node_text(data.getElementsByTagName(source)[0].childNodes)

                # as API does not return only Tier-0 sites, we have forced the
                # sitename to CERN-PROD to filter the list
                if group == 'tier0' and region != sitename:
                    continue

                # as API does not return only Tier-1 sites, we filter the
                # Tier-0 and show the rest of the list
                elif group == 'tier1' and region == 'CH-CERN':
                    continue

                # this is not the site or country we are looking for
                elif sitename not in (region, 'all'):
                    continue

                if view == PLOT_PERC_HOURS_USED:
                    # this plot needs to provide an average value. For this,
                    # all vo values are collected. Later on, the average value
                    # will be calculated along with the sum of the vo pledges.
                    if sitename in (region, 'all'):
                        for vidx, vname in enumerate(vos_lower):
                            if data_vo in (vname, 'local-%s' % vname):
                                if pdata[year][vname]:
                                    cdata = get_node_text(data.getElementsByTagName(query1)[0].childNodes) or 0.0
                                    try:
                                        cdata = float(cdata)
                                    except ValueError:
                                        cdata = 0.0
                                    caldata = round(((cdata * factor) / pdata[year][vname]) * 100, 2)
                                    idx = vidx + 1  # item 0 is month-year, then VO values follow
                                    results['mapdata']['rows'][j][idx] += caldata
                                    resulted_regions.append(region)

                                    # add value for average calculation
                                    tcdata += cdata
                                    if vname not in pledge_added_vos:
                                        pledge_added_vos.add(vname)

                elif view == PLOT_CPU_EFFICENCY:
                    # this plot needs to gather *two values* and make a division
                    # to obtain its results, so in this stage only the values
                    # are gathered. The formula will be applied later on, when
                    # the values are mapped to the results.
                    if data_vo in (voname, 'local-%s' % voname):
                        if region not in regions:
                            regions[region] = icount
                            ndatas.append([0.0, 0.0])
                            icount += 1
                        cpu_query, wct_query = query1
                        cpu_cdata = get_node_text(data.getElementsByTagName(cpu_query)[0].childNodes) or 0.0
                        wct_cdata = get_node_text(data.getElementsByTagName(wct_query)[0].childNodes) or 0.0
                        try:
                            cpu_cdata = float(cpu_cdata)
                            wct_cdata = float(wct_cdata)
                        except ValueError:
                            cpu_cdata = 0.0
                            wct_cdata = 0.0
                        cpu, wct = ndatas[regions[region]]
                        ndatas[regions[region]] = [cpu + cpu_cdata, wct + wct_cdata]
                        resulted_regions.append(region)

                # other plots
                else:
                    if data_vo in (voname, 'local-%s' % voname):
                        if region not in regions:
                            regions[region] = icount
                            ndatas.append(0)
                            icount += 1
                        cdata = get_node_text(data.getElementsByTagName(query1)[0].childNodes) or 0.0
                        try:
                            cdata = float(cdata)
                        except ValueError:
                            cdata = 0.0
                        ndatas[regions[region]] += round(cdata * factor, 2)
                        resulted_regions.append(region)

            # adjust values and map them to graph data structure
            if len(resulted_regions):
                if view == PLOT_PERC_HOURS_USED:
                    # calculate average value
                    tpdata = sum(pdata[year][v] for v in pledge_added_vos)
                    if tpdata > 0:
                        tdata = round(((tcdata * factor) / tpdata) * 100, 2)
                    results['mapdata']['rows'][j].append(tdata)

                elif view == PLOT_CPU_EFFICENCY:
                    for cpu, wct in ndatas:
                        value = round((cpu / (wct or 1)) * factor, 2)
                        results['mapdata']['rows'][j].append(value)
                    # clear values for next month
                    ndatas = [[0.0, 0.0] for n in ndatas]

                else:
                    for ndata in ndatas:
                        results['mapdata']['rows'][j].append(ndata)
                    # clear values for next month
                    ndatas = [0.0 for n in ndatas]
            else:
                # add an empty value to plot the empty graph
                results['mapdata']['rows'][j].append(0)

            j = j + 1
            stime += relativedelta(months=+1)


        if view == PLOT_PERC_HOURS_USED:
            for vo in vos + ['Total']:
                results['mapdata']['cols'].append({
                    'label': vo,
                    'type': 'number'
                })
        else:
            if resulted_regions:
                for name in sorted(regions.items(), key=lambda r: r[1]):
                    results['mapdata']['cols'].append({
                        'label': name[0],
                        'type': 'number'
                    })
            else:
                # add empty column to plot the empty graph
                results['mapdata']['cols'].append({
                    'type': 'number', 'label': u'No data'
                })

    if csv:
        if tab == 'eu':
            filename = 'Experiments_Usage_Trends.csv'
        elif view == PLOT_AVAILABILITY:
            filename = 'Availability_Trends.csv'
        else:
            filename = 'Reliability_Trends.csv'
        template = loader.get_template('trends_csv')
        ctx = Context({'results': results['mapdata']})
        response = HttpResponse(mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=%s' % filename
        response.write(template.render(ctx))
        return response

    data = simplejson.dumps(results)
    return HttpResponse(data, mimetype='application/json')
