from django.conf.urls.defaults import *
from os import path as os_path
from django.conf import settings


handler400 = 'mywlcg.base.views.error_400'
handler404 = 'mywlcg.base.views.error_404'
handler500 = 'mywlcg.base.views.error_500'


urlpatterns = patterns('',
    (r'^$', 'mywlcg.base.views.index'),
    (r'^blog/$', 'mywlcg.base.views.blog'),
    (r'^media/(.*)', 'django.views.static.serve', 
		{'document_root': os_path.join(settings.APP_PATH, '../media')}),
    (r'^services/$', 'mywlcg.services.views.index'),
    (r'^services/(?P<site_id>[0-9,]+)/(?P<profile_id>[0-9-]+)/(?P<unixtime>[0-9]+)/$', 
		'mywlcg.services.views.selected'),
    (r'^services/selected_json/(?P<site_id>[0-9,]+)/(?P<profile_id>[0-9-]+)/'\
		+'(?P<unixtime>[0-9]+)/$', 'mywlcg.services.views.selected_json'),
    (r'^services/json_datatables/$', 'mywlcg.services.views.json_datatables'),
    (r'^services/json/$', 'mywlcg.services.views.json'),
    (r'^services/csv/$', 'mywlcg.services.views.csv'),
    (r'^services/xml/$', 'mywlcg.services.views.xml'),
    (r'^services/summary/$', 'mywlcg.services.views.status_summary'),
    (r'^status/$', 'mywlcg.status.views.index'),
    (r'^status/(?P<service_id>[0-9,]+)/(?P<profile_id>[0-9-]+)/(?P<unixtime>[0-9]+)/$', 
		'mywlcg.status.views.selected'),
    (r'^status/json_datatables/$', 'mywlcg.status.views.json_datatables'),
    (r'^status/json/$', 'mywlcg.status.views.json'),
    (r'^status/csv/$', 'mywlcg.status.views.csv'),
    (r'^status/xml/$', 'mywlcg.status.views.xml'),
    (r'^ss/$', 'mywlcg.ss.views.index'),
    (r'^ss/json/$', 'mywlcg.ss.views.json'),
    (r'^ss/graph/(?P<service_id>[0-9]+)/(?P<profile_id>[0-9-]+)/(?P<starttime>[0-9]+)/'\
		+'(?P<endtime>[0-9]+)/$', 'mywlcg.ss.views.graph'),
    (r'^ss/(?P<hostname>.+)/(?P<flavour>.+)/$', 'mywlcg.ss.views.one'),
    (r'^ss/(?P<hostname>.+)/(?P<flavour>.+)/(?P<startdate>[0-9-]+)/$', 'mywlcg.ss.views.one'),
    (r'^ss/(?P<hostname>.+)/(?P<flavour>.+)/(?P<startdate>[0-9-]+)/(?P<enddate>[0-9-]+)/$', 
		'mywlcg.ss.views.one'),
    (r'^sa/$', 'mywlcg.sa.views.index'),
    (r'^sa/json/$', 'mywlcg.sa.views.json'),
    (r'^sa/heading/$', 'mywlcg.sa.views.heading'),
    (r'^metric/$', 'mywlcg.metric.views.index'),
    (r'^metric/json/$', 'mywlcg.metric.views.json'),
    (r'^metric/(?P<metricdata_id>[0-9]+)/$', 'mywlcg.metric.views.metricdata'),
    (r'^metric/(?P<service_id>[0-9]+)/(?P<metric_id>[0-9]+)/(?P<timestamp>[0-9]+)/$', 
		'mywlcg.metric.views.metricdata2'),
    (r'^metric/popup/(?P<vo_id>[0-9]+)/(?P<service_id>[0-9]+)/(?P<metric_id>[0-9]+)/'\
		+'(?P<timestamp>[0-9]+)/$', 'mywlcg.metric.views.metricdata2popup'),
    (r'^metric/latest/(?P<metricdata_id>[0-9]+)/$', 'mywlcg.metric.views.metricdatalatest'),
    (r'^metric/latest/popup/(?P<metricdata_id>[0-9]+)/$', 
		'mywlcg.metric.views.metricdatalatestpopup'),
    (r'^tests/$', 'mywlcg.tests.views.index'),
    (r'^tests/results/$', 'mywlcg.tests.views.results'),
    (r'^tests/results/(?P<type>html|text|nagios)/$', 'mywlcg.tests.views.results'),
    (r'', include('mywlcg-atp-web.urls_atp')),
)

# filters
urlpatterns += patterns('',
    (r'^filters/json/(?P<type>Groups|Sites|profiles|Hostnames|Regions|Flavours|Regions_or_Tiers|Physical_Sites)/(?P<vo>[0-9]+)/$', 
		'mywlcg.filters.views.json_group'),
    (r'^dt/filters/json/(?P<type>Groups|Sites|profiles|Hostnames|VOs|Source_Sites|'\
		+'Destination_Sites)/(?P<vo>[0-9]+)/$', 'mywlcg.filters.views.json_group'),
    (r'^trends/filters/json/(?P<type>Groups|Sites|profiles|Hostnames)/(?P<vo>[0-9]+)/$', 
		'mywlcg.filters.views.json_group'),
)

# gridmap
urlpatterns += patterns('mywlcg.gridmap.views',
    (r'^gridmap/$', 'index'),
    (r'^gridmap/json/$', 'json'),
    (r'^gridmap/onshow/$', 'onshow'),
    (r'^gridmap/onshowsize/$', 'onshowsize'),
)

# reports
if settings.REPORTS:
    urlpatterns += patterns('',
        (r'^reports/', include('mywlcg.reports.urls')),
    )

# data transfers
if settings.DAX:
    urlpatterns += patterns('mywlcg.dt.views',
        url(r'^dt/$', 'index'),
        url(r'^dt/(?P<tab>\w{2})/$', 'tab', name="data-transfers"),
        url(r'^dt/(?P<tab>\w{2})/heading/$', 'heading'),
        url(r'^dt/(?P<tab>\w{2})/json/$', 'json'),
    )

# trends
if settings.TRENDS:
    urlpatterns += patterns('mywlcg.trends.views',
        url(r'^trends/$', 'index'),
        url(r'^trends/(?P<tab>\w{2})/$', 'tab', name="trends"),
        url(r'^trends/(?P<tab>\w{2})/heading/$', 'heading'),
        url(r'^trends/(?P<tab>\w{2})/json/$', 'json'),
    
    )

# web service
urlpatterns += patterns('', 
    (r'^sam-pi/', include('mywlcg.ws.urls')),
    (r'^sam-pi/', include('mywlcg-atp-api.urls')),
)
