from datetime import datetime,timedelta
import re
from django.shortcuts import render_to_response
from django.conf import settings
from django.core.cache import cache
from django.utils.functional import wraps
from django.http import HttpResponseBadRequest
from mywlcg import settings

DB_LIMIT_ERROR="400: Bad Request. OOPS!!! YOU ARE EXCEEDING THE DB QUERY LIMIT. "\
			+ " PLEASE CONTACT THE ADMINISTRATOR FOR FURTHER INFORMATION."

ACCESS_LIMIT_ERROR="400: Bad Request. OOPS!!! YOU ARE EXCEEDING THE NUMBER OF ACCESS ALLOWED "\
			+ "PER USER IN A GIVEN TIME. "\
			+ " PLEASE CONTACT THE ADMINISTRATOR FOR FURTHER INFORMATION."

PARAMETER_MISSING_ERROR="400: Bad Request. PLEASE PROVIDE ALL THE MANDATORY INPUT PARAMETERS"

def throttle(limit=3, duration=900):
    """
    Throttle the given function, returning error if access limit exceeded
    
    Example::
    
        # limit to 5 requests per 5 minutes:
        
        @throttle(5, 300)
        def my_view(request, ...):
            # do some stuff
    
    
        # limit to 3 POST requests per 15 minutes:
    
        @throttle
        def my_other_view(request, ...):
            # ..self.
    """
    
    def decorator(func):
        @wraps(func)
        def inner(request, *args, **kwargs):
          if settings.THROTTLE:
            ident = request.META.get('REMOTE_ADDR', None)
            output=request.GET.get('output') or 'xml'
            now = datetime.now()
            ts_key = 'throttle:ts:%s' % ident
            timestamp = cache.get(ts_key)
            offset = now + timedelta(seconds=duration)
            if timestamp and timestamp < offset:
                wait = duration - (offset-timestamp).seconds
                cache.set(ts_key, timestamp, wait)
                return HttpResponseBadRequest(error(output,2,wait))
            count = cache.get(ident, 1)
            cache.set(ident, count+1)
            
            if count >= limit:
                cache.set(ts_key, offset, duration)
                cache.set(ident, 1)
              
          return func(request, *args, **kwargs)
        return inner
    
    return decorator

##########################################################
# Return error
def error(output,err,wait=None):
    if err==1:
        text=DB_LIMIT_ERROR
    elif err==2:
	text=ACCESS_LIMIT_ERROR + " PLEASE WAIT FOR " + str(wait) + " SECONDS."
    elif err==3:
        text=PARAMETER_MISSING_ERROR
    else:
        text=err 
    if output == "json":
	return render_to_response(
		'ws_error_json',
		{'error': text},
		mimetype = 'application/javascript',
	)
    else:
	return render_to_response(
		'ws_error_xml',
		{'error': text},
		mimetype = 'application/xml',
	)
