// google charts package must be the first thing to load
google.load('visualization', '1.0', {'packages':['corechart']});

month_abbr = Array(
    'jan', 'feb', 'mar', 'apr', 'may', 'jun',
    'jul', 'ago', 'sep', 'oct', 'nov', 'dec'
);

// vos in which each site is present
site_vos = undefined;

function select_month_range(e){
    // highlight only months after the target
    selected = $(e.target);
    months = $('span.period-month');
    selected_idx = months.index(selected);
    months.each(function(idx, m){
        if (idx < selected_idx){
            $(m).removeClass('active');
        }else{
            $(m).addClass('active');
        }
    });
}

function restore_month_range(e){
    // highlight only months after the saved active month
    months = $('span.period-month');
    months.each(function(idx, m){
        if (idx < selidx){
            $(m).removeClass('active');
        }else{
            $(m).addClass('active');
        }
    });
}

function set_month_range(e){
    // change the active month to preserve selection
    selected = $(e.target);
    months = $('span.period-month');
    selected_idx = months.index(selected);
    selidx = selected_idx;
    $('#reports-period-history').val(nmonths - selidx - 1);
}

function update_month_range(e, date){
    // update month options to adapt to new selected month
    if (!date){
        date = new Date($('#reports-period-month').val());
    }
    y = date.getFullYear();
    m = date.getMonth();
    months = $('span.period-month');
    // set year and month for first month
    $(months[nmonths - 1]).text(month_abbr[m--] + ' ' + y);
    // set only month for the rest
    for (var i = nmonths - 2; i >= 0; i--){
        if (m < 0){
            m = 11;
            y -= 1;
        }
        $(months[i]).attr('rel', y + '-' + (m + 1));  // js months start with 0
        $(months[i]).text(month_abbr[m--]);
    }
}

function show_graph(dataurl, el){
    // retrieves the data through AJAX and displays a google chart

    function round(v){
        if (v == null)
            return null;
        return Math.round(v*100) / 100;
    }

    function show_google_chart(avl_data, container){
        var data = new google.visualization.DataTable();
        data.addColumn('string', 'Date');
        data.addColumn('number', 'Availability');
        rows = Array();
        for (var i = 0; i < avl_data.length; i++){
            rows.push([avl_data[i][0], round(avl_data[i][1])]);
        }
        data.addRows(rows);

        title = $(container).find('span.title').text();
        values = $(container).find('span.values').text();
        plot_width = $(container).find('span.width').text();
        plot_width = plot_width ? plot_width : 232;
        color = (values == 'reliability') ? 'blue' : 'green';
        psize = (avl_data.length >= 31) ? ((avl_data.length >= 60) ? 0 : 1) : 2;

        var chart = new google.visualization.AreaChart(container);
        chart.draw(data, {
            title: title,
            legend: 'none',
            width: plot_width,
            height: 160,
            chartArea: {width: plot_width * 1, left: 30},
            pointSize: psize,
            vAxis: { 
                viewWindow:{
                    min: 0, max: 105
                }
            },
            series: [{color: color}]
        })
    }

    $.ajax({
        url: dataurl,
        dataType: 'json',
        success: function(avl_data, status, jqXHR){
            // fail if avl_data has error
            $(el).find('img').remove();
            $(el).css('border', 'none');
            show_google_chart(avl_data, el);
        },
        error: function(jqXHR, status, error){
            $(el).find('img').remove();
            console.log(status, error);
        }
    });
}

$(document).ready(function(){
    google.setOnLoadCallback(function(){
        init_ui();
    });
});

function init_ui(){
    // highlight top level navigation
    $('#reports').addClass('menuselected');

    // select defined month
    nmonths = $('span.period-month').length;
    rephistory = $('#reports-period-history').val();
    selidx = (nmonths - 1) - parseInt(rephistory);
    restore_month_range(null);

    // fill month selector with proper dates
    selected_month = $('#reports-period-month').val();
    if (selected_month == 'latest'){
        update_month_range(null, new Date());
        $(months[nmonths - 1]).text('last 30 days');
    }else{
        month_year = selected_month.split('-');
        y = Math.round(month_year[0]);
        m = Math.round(month_year[1]);
        update_month_range(null, new Date(y, m-1));
    }

    // enable month selector
    $('span.period-month').bind('mouseover', select_month_range);
    $('span.period-month').bind('mouseout', restore_month_range);
    $('span.period-month').bind('click', set_month_range);

    // activate monthpicker
    date = new Date();
    year = date.getFullYear();
    month = date.getMonth();
    $('#reports-period-month').monthpicker({
        id: 'monthpicker_period',
        pattern: 'yyyy-mm',
        startYear: 2006,
        finalYear: year
    });

    // enable monthpicker toggling
    $('#reports-period-pick-month').click(function(e){
        $('#reports-period-month').monthpicker('show');
        $('#monthpicker_period').css({
            left: $(e.target).offset().left,
            top: $(e.target).offset().top + 30
        })
    });
    $("#reports-period-month").monthpicker().bind('monthpicker-click-month', function(e, month){
        update_month_range();
    });

    // activate last month button
    $('#reports-period-pick-latest').click(function(e){
        update_month_range(null, new Date());
        $('#reports-period-month').val('latest');
        months = $('span.period-month');
        $(months[nmonths - 1]).text('last 30 days');
        // restore history
        selidx = (nmonths - 1);
        $('#reports-period-history').val(0);
        restore_month_range(null);
    });

    // populate filters
    site_vos_url = $('#view-selection').attr('rel');
    selected_vos = ($('#vos').val() == 'all') ? 'all' : $('#vos').val().split(',');
    selected_sites = ($('#sites').val() == 'all') ? 'all' : $('#sites').val().split(',');
    $.ajax({
        url: site_vos_url,
        dataType: 'json',
        success: function(data, status, jqXHR){

            function option(name, id, value, selected, fn){
                item = $('<li><input type="checkbox" name="" rel=""><label></label>');
                item.find('input').attr('id', id).attr('name', name).attr('rel', value);
                item.find('label').attr('for', id).text(value.toUpperCase());
                if ($.inArray(value, selected) != -1 || selected == 'all'){
                    item.find('input').attr('checked', 'checked');
                }
                if (fn){
                    item.find('input').change(function(e){
                        fn(value, e);
                    });
                }
                return item;
            }

            function select_vo_sites(vo, e){
                if ($(e.target).attr('checked')){
                    $.each(site_vos, function(idx, site_volist){
                        $.each(site_volist, function(sitename, vos){
                            if ($.inArray(vo, vos) != -1){
                                $('#site_' + sitename).attr('checked', 'checked');
                            }
                        });
                    });
                }
            }

            site_vos = data;
            found_vos = Array();
            volist = Array();
            site_list = $('.site-list');
            $.each(site_vos, function(idx, site_volist){
                // populate VO / Site list
                $.each(site_volist, function(sitename, vos){
                    $.each(vos, function(idx, voname){
                        if ($.inArray(voname, found_vos) == -1){
                            found_vos.push(voname);
                            volist.push(option('view_vo', 'vo_' + voname, voname, selected_vos, select_vo_sites));
                        }
                    });

                    site_list.append(option('view_site', 'site_' + sitename, sitename, selected_sites));

                    // group in lists of 10 items
                    if (site_list.children().length > 8){
                        new_list = $('<ul class="site-list"></ul>');
                        new_list.insertAfter(site_list);
                        site_list = new_list;
                    }
                });
            });

            // order VO list A-Z
            volist.sort(function(a, b){
                a = $(a).find('input').attr('rel');
                b = $(b).find('input').attr('rel');
                return (a > b) ? 1 : ((a < b) ? -1 : 0);
            });
            $.each(volist, function(idx, item){
                $('#vo-list').append(item);
            });

            $('#view-selection').find('li.loading').remove();
        },
        error: function(jqXHR, status, error){
        }
    });

    // bind filters display with form
    $('#view-groupby').find('input').change(function(e){
        $('#groupby').val($(e.target).val());
        return e;
    })

    // enable select/unselect all
    function set_and_trigger(value, el){
        $(el.target).closest('div.select-section').find('input[type="checkbox"]').each(function(idx, el){
            if ($(el).attr('checked') != value){
                $(el).attr('checked', value).trigger('change');
            }
        });
    }
    $('.select-all').click(function(el){ set_and_trigger('checked', el); });
    $('.deselect-all').click(function(el){ set_and_trigger('', el); });

    function save_filters(){
        function list_or_all(list_el){
            all_items = list_el.find('input[type="checkbox"]');
            selected_items = list_el.find('input[type="checkbox"]:checked');
            if (all_items.length == selected_items.length){
                return 'all';
            }else{
                who = Array();
                $.each(selected_items, function(idx, w){
                    who.push($(w).attr('rel'));
                });
                return who.join(',');
            }
        }
        $('#vos').val(list_or_all($('#vo-list')));
        $('#sites').val(list_or_all($('.site-list')));
    }

    $('#select_reload').click(function(el){
        save_filters();
        $('form').submit();
    });
    $('#select_selection').click(function(el){
        save_filters();
        $('#select_view').dialog('close');
    });

    $('#reports-view').click(function(){
        $('#select_view').dialog({
            modal: true,
            resizable: false,
            width: 'auto',
            height: 'auto'
        });
    });

    // display availability plots
    $('.availability-plot').each(function(idx, el){
        show_graph($(el).attr('rel'), el);
    });

}
